import logconfig
import numpy as np
import os
import auxiliary as aux
import txantdiag_plotdata as plt

myLogger = logconfig.txAntDiagLogger.logger



# Get the list of all configred graphs to be drawn.
def getGraphList(graphConfig):
    graphList = []
    for graph, set in graphConfig.items():
        if set == 1:
            graphList.append(graph)
    return graphList

# Date time and sensor combination list
def getDateTimeSensorCombList(parseDataList):
    outputDictList = []
    sensor = ''
    dateAndTime = ''
    for listIdx in range(len(parseDataList)):
        outputDict = dict()

        if parseDataList[listIdx].fileTitle.sensorFound != 0 and parseDataList[listIdx].fileTitle.dateAndTimeFound != 0:
            if parseDataList[listIdx].fileTitle.sensor != sensor or parseDataList[listIdx].fileTitle.dateAndTime != dateAndTime:
                dateAndTime = parseDataList[listIdx].fileTitle.dateAndTime
                sensor = parseDataList[listIdx].fileTitle.sensor
                outputDict.update({"dateAndTime":dateAndTime,"sensor":sensor})
                outputDictList.append(outputDict)
    return outputDictList

def getTxAntList(parseDataList):
    txAntList = []

    for pdIdx in range(len(parseDataList)):
        if parseDataList[pdIdx].fileTitle.txAntFound == 1:
            txAnt = parseDataList[pdIdx].fileTitle.txAnt
            txAntList.append(txAnt)

    txAntSet  = set(txAntList)

    txAntList = list(txAntSet)

    return txAntList


# Mapping string for the antenna diagram types
def getAntDiagType(inputString):
    if inputString == 'Az':
        antennaDiagType = 'Azimuth'
    else:
        antennaDiagType = 'Elevation'
    return antennaDiagType

def fillGraphSlides(presentation, slideLayout, graphTxAntList, centFreqDictList, diagType, normAxisAvgAngleList, plotSetup):

    shapeCols=slideLayout.getNofGraphs()

    for graphTxAntIdx in range(len(graphTxAntList)):

        graphTxAntDict = graphTxAntList[graphTxAntIdx]

        txAntPerGraphList = graphTxAntDict['orig']
        txAntPerNormGraphList = graphTxAntDict['norm']

        if len(txAntPerGraphList) != 0 or len(txAntPerNormGraphList) != 0:

            nofTxAnt = len(txAntPerNormGraphList)
            if len(txAntPerGraphList) != 0:
                nofTxAnt = len(txAntPerGraphList)

            if plotSetup['plotOrigGraphs'] == 1:
                normGraphs = 0

                txAntGraphData = txAntPerGraphList

                res = nofTxAnt % shapeCols

                if nofTxAnt < shapeCols:
                    nofShapeElements = shapeCols
                    fillNum = shapeCols - nofTxAnt
                    for i in range(fillNum):
                        txAntGraphData.append('')
                elif res == 0:
                    nofShapeElements = nofTxAnt
                else:
                    fillNum = shapeCols -res
                    nofShapeElements = nofTxAnt + fillNum
                    for i in range(fillNum):
                        txAntGraphData.append('')

                shapeRows = int(nofShapeElements / shapeCols)

                txAntGraphData = np.reshape(txAntGraphData, (shapeRows, shapeCols))

                for txAntGraphDataRow in range(shapeRows):
                    # Adding a slide with one graph layout
                    presentation.add_slide(slideLayout)

                    textTxAntString = ''

                    imageList = []
                    captionsList = []
                    freqString = ''

                    # Filling the caption and image lists
                    for txAntGraphDataCol in range(shapeCols):
                        if txAntGraphData[txAntGraphDataRow][txAntGraphDataCol] != '':
                            img = slideLayout.imageList[txAntGraphDataCol]
                            cap = slideLayout.captionsList[txAntGraphDataCol]
                            imageList.append(img)
                            captionsList.append(cap)


                    newLineStr = '\n'

                    for txAntGraphDataCol in range(shapeCols):

                        graphData = txAntGraphData[txAntGraphDataRow][txAntGraphDataCol]

                        if graphData != '':

                            if diagType == 'Az':
                                normAxis = 'El'
                            else:
                                normAxis = 'Az'

                            textTxAntString += newLineStr + ' - TxAnt' + str(graphData.txAnt) + ': Freq. = ' + str(centFreqDictList[txAntGraphDataCol]["avgFreqGHz"]) + ' GHz (+ ' + str(centFreqDictList[txAntGraphDataCol]["posFreqDevMHz"]) + ' MHz - ' + str(centFreqDictList[txAntGraphDataCol]["negFreqDevMHz"]) + ' MHz), '
                            textTxAntString += normAxis+'= '+normAxisAvgAngleList[txAntGraphDataCol][diagType]+'°'
                            imgPath = os.path.join(aux.txAntDiagOutputPath, graphData.figureTitle)
                            imageList[txAntGraphDataCol].insert_picture(imgPath)

                            captionsList[txAntGraphDataCol].text_frame.text = 'TxAnt' + str(graphData.txAnt)

                    # Inserting slide title.
                    slideLayout.title.text = plt.graphStringMapping(txAntGraphData[txAntGraphDataRow][0].graphTitle,normGraphs) + ' - Tx Antenna Diagrams - Type ' \
                                             + ' - ' + getAntDiagType(diagType)

                    # Inserting the description text
                    slideLayout.text.text_frame.text = plt.graphStringMapping(txAntGraphData[txAntGraphDataRow][0].graphTitle,normGraphs) + ' - Tx Antenna Diagrams - ' + \
                                                    getAntDiagType(diagType) + textTxAntString  \
                                                    + newLineStr +' - All sensors.'

            # Insert normalised Graph Slides
            if plotSetup['plotNormGraphs'] == 1:
                normGraphs = 1

                txAntGraphData = txAntPerNormGraphList

                insertSlides = 1
                for i in range(len(txAntGraphData)):
                    if txAntGraphData[i].graphTitle == 'frequency' or txAntGraphData[i].graphTitle =='time':
                        insertSlides = 0

                if insertSlides == 1:

                    res = nofTxAnt % shapeCols

                    if nofTxAnt < shapeCols:
                        nofShapeElements = shapeCols
                        fillNum = shapeCols - nofTxAnt
                        for i in range(fillNum):
                            txAntGraphData.append('')
                    elif res == 0:
                        nofShapeElements = nofTxAnt
                    else:
                        fillNum = shapeCols - res
                        nofShapeElements = nofTxAnt + fillNum
                        for i in range(fillNum):
                            txAntGraphData.append('')

                    shapeRows = int(nofShapeElements / shapeCols)

                    txAntGraphData = np.reshape(txAntGraphData, (shapeRows, shapeCols))

                    for txAntGraphDataRow in range(shapeRows):

                        # Adding a slide with one graph layout
                        presentation.add_slide(slideLayout)

                        textTxAntString = ''

                        imageList = []
                        captionsList = []
                        freqString = ''

                        # Filling the caption and image lists
                        for txAntGraphDataCol in range(shapeCols):
                            if txAntGraphData[txAntGraphDataRow][txAntGraphDataCol] != '':
                                img = slideLayout.imageList[txAntGraphDataCol]
                                cap = slideLayout.captionsList[txAntGraphDataCol]
                                imageList.append(img)
                                captionsList.append(cap)

                        newLineStr = '\n'



                        for txAntGraphDataCol in range(shapeCols):

                            graphData = txAntGraphData[txAntGraphDataRow][txAntGraphDataCol]



                            if graphData != '':

                                if diagType == 'Az':
                                    normAxis = 'El'
                                else:
                                    normAxis = 'Az'

                                textTxAntString += newLineStr + ' - TxAnt' + str(graphData.txAnt) + ': Freq. = ' + str(
                                    centFreqDictList[txAntGraphDataCol]["avgFreqGHz"]) + ' GHz (+ ' + str(
                                    centFreqDictList[txAntGraphDataCol]["posFreqDevMHz"]) + ' MHz - ' + str(
                                    centFreqDictList[txAntGraphDataCol]["negFreqDevMHz"]) + ' MHz), '
                                textTxAntString += normAxis + '= ' + normAxisAvgAngleList[txAntGraphDataCol][diagType] + '°'
                                imgPath = os.path.join(aux.txAntDiagOutputPath, graphData.figureTitle)
                                imageList[txAntGraphDataCol].insert_picture(imgPath)

                                captionsList[txAntGraphDataCol].text_frame.text = 'TxAnt' + str(graphData.txAnt)

                        # Inserting slide title.
                        slideLayout.title.text = plt.graphStringMapping(txAntGraphData[txAntGraphDataRow][0].graphTitle,normGraphs) + ' - Tx Antenna Diagrams - Type ' \
                                                 + ' - ' + getAntDiagType(diagType)

                        # Inserting the description text
                        slideLayout.text.text_frame.text = plt.graphStringMapping(txAntGraphData[txAntGraphDataRow][0].graphTitle,normGraphs) + ' - Tx Antenna Diagrams - ' + \
                                                           getAntDiagType(diagType) + textTxAntString \
                                                           + newLineStr + ' - All sensors.'


def getFreqListPerTxAnt(txAnt, parseDataList):
    freqList = []

    for pdIdx in range(len(parseDataList)):
        if parseDataList[pdIdx].fileTitle.txAntFound==1:
            if txAnt == parseDataList[pdIdx].fileTitle.txAnt:
                freq = parseDataList[pdIdx].ptuMaxSpecHeaderData.frequenz_MHz
                freqList.append(freq)
    return freqList


def getAvgFreqPerTxAnt(txAnt, parseDataList):
    freqList = getFreqListPerTxAnt(txAnt, parseDataList)

    avgFreq = 0
    if len(freqList) != 0:
        avgFreq = sum(freqList)/len(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return avgFreq


def getMaxFreqPerTxAnt(txAnt, parseDataList):
    freqList = getFreqListPerTxAnt(txAnt,parseDataList)
    maxFreq  = 0
    if len(freqList) != 0:
        maxFreq = max(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return maxFreq

def getMinFreqPerTxAnt(txAnt, parseDataList):
    freqList = getFreqListPerTxAnt(txAnt, parseDataList)
    minFreq  = 0
    if len(freqList) != 0:
        minFreq = min(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return minFreq

# Freq functions, all parsed tx ant diagrams
def getFreqListAll(parseDataList):
    freqList = []

    for pdIdx in range(len(parseDataList)):
        freq = parseDataList[pdIdx].ptuMaxSpecHeaderData.frequenz_MHz
        freqList.append(freq)
    return freqList


def getAvgFreqAll(parseDataList):
    freqList = getFreqListAll(parseDataList)

    avgFreq  = 0
    if len(freqList) != 0:
        avgFreq = sum(freqList)/len(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return avgFreq


def getMaxFreqAll(parseDataList):
    freqList = getFreqListAll(parseDataList)

    maxFreq = 0
    if len(freqList) != 0:
        maxFreq = max(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return maxFreq

def getMinFreqAll(parseDataList):
    freqList = getFreqListAll(parseDataList)
    minFreq  = 0
    if len(freqList) != 0:
        minFreq = min(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return minFreq

# Normal axis average angle per tx antenna
def getNormAxisAvgAngPerTxAnt(txAnt, parseDataList, diagType):
    normAxisAngList = []
    for pdIdx in range(len(parseDataList)):
        if parseDataList[pdIdx].fileTitle.txAntFound == 1:
            if txAnt == parseDataList[pdIdx].fileTitle.txAnt:

                if diagType == 'Az':
                    normAxisAng = parseDataList[pdIdx].ptuMaxSpecHeaderData.d_PTU_ElAngle
                else:
                    normAxisAng = parseDataList[pdIdx].ptuMaxSpecHeaderData.d_PTU_AzAngle

                normAxisAngList.append(normAxisAng)

    avgAng = 0
    if len(normAxisAngList) != 0:
        avgAng = sum(normAxisAngList) / len(normAxisAngList)
    else:
        myLogger.error("Empty frequency list.")

    return avgAng




import os
import shutil
from shutil import copyfile
import errno
import logconfig
import enum
import sys

class OutputFolderNames(enum.Enum):
    AntennaSystemDiagrams='output'
    TxAntDiagrams='output_tx_diagrams'
    AntennaPatterns='output_antenna_patterns'
    HuellKurven = 'output_huellkurven'
    Certificates='output_certificates'
    SetCurves='output_setcurves'
    Statistic='output_statistic'
    merge= 'output_merge'


# Get logger
myLogger=logconfig.mainLoadDataLogger.logger

# Basic directory
# basicDir = os.path.dirname(os.path.abspath(__file__))
basicDir = os.path.dirname(sys.argv[0])

# Output pazh
antSysDiagOutputPath = os.path.join(basicDir, OutputFolderNames.AntennaSystemDiagrams.value)
antPatOutputPath = os.path.join(basicDir, OutputFolderNames.AntennaPatterns.value)
huellKurvenOutputPath = os.path.join(basicDir, OutputFolderNames.HuellKurven.value)
txAntDiagOutputPath = os.path.join(basicDir, OutputFolderNames.TxAntDiagrams.value)
setCurvesOutputPath = os.path.join(basicDir, OutputFolderNames.SetCurves.value)
# mainLoadDataOutputPath = os.path.join(basicDir, OutputFolderNames.LoadData.value)
certificatesOutputPath = os.path.join(basicDir, OutputFolderNames.Certificates.value)
statisticOutputPath = os.path.join(basicDir, OutputFolderNames.Statistic.value)
mergeOutputPath = os.path.join(basicDir, OutputFolderNames.merge.value)

#setcurves
setcurvesinputconffilename = 'SetCurvesInputConf.json'
setcurvesinputconffilepath = os.path.join(basicDir, setcurvesinputconffilename)
setcurvescustinputconffilename = 'SetCurvesCustomerInputConf.json'
setcurvescustinputconffilepath = os.path.join(basicDir, setcurvescustinputconffilename)

#stats input config
statsinputconffilename = 'StatisticsLimitsConfig.json'
statsinputconffilepath = os.path.join(basicDir, statsinputconffilename)

burnInLimitsFileName = 'BurnInLimits.yaml'
burnInLimitsFilePath = os.path.join(basicDir, burnInLimitsFileName)

calLimitsFileName = 'CalLimits.yaml'
calLimitsFilePath = os.path.join(basicDir, calLimitsFileName)

burnInLimGroupsFileName = 'BurnInLimGroups.yaml'
burnInLimGroupsFilePath = os.path.join(basicDir, burnInLimGroupsFileName)

calLimGroupsFileName = 'CalLimGroups.yaml'
calLimGroupsFilePath = os.path.join(basicDir, calLimGroupsFileName)

# Clear the output directory
def clearAll(outputFolderPath):
    status = True
    errorString = 'Clearing output directory: Status OK.'
    try:
        for the_file in os.listdir(outputFolderPath):
            file_path = os.path.join(outputFolderPath, the_file)
            try:
                if os.path.isfile(file_path):
                    os.unlink(file_path)
                elif os.path.isdir(file_path):
                    shutil.rmtree(file_path)
            except Exception as e:
                errorString = "Clearing output directory: " + str(e)
                status=False
    except Exception as e:
        errorString = "Clearing output directory: " + str(e)
    return status,errorString

def createDirectory(outputPath):
    errorStr = 'Creating directory OK, path: '+ outputPath +'.'
    status=True
    if not os.path.isdir(outputPath):
        try:
            os.makedirs(outputPath)
            # myLogger.info(errorStr)
        except OSError as exc:  # Guard against race condition
            if exc.errno != errno.EEXIST:
                errorStr = str(exc.errno)
                status=False
                # myLogger.error(errorStr)
    return status, errorStr

def copyFile(src,dest):
    errorStr='Copying: Status OK.'
    status = True
    try:
        copyfile(src,dest)
    except Exception as e:
        errorStr='Copying: '+str(e)
        status=False

    return status,errorStr

# Save the plotted graph figure with a given title.
def saveFig(fig,figurePath):
    status=True
    errorStr=''
    figurePath=figurePath+'.png'
    fig.set_figheight(9)
    fig.set_figwidth(16)
    figurePath=os.path.join(figurePath,figurePath)
    try:
        fig.savefig(figurePath, dpi=150)
    except Exception as e:
        errorStr='Saving figure: '+str(e)
        status=False
    return status, errorStr



import enum
import pptx
import sys
from abc import ABC, abstractmethod
import os
import numpy as np
import sys


# Mapping slide layouts
class SlideLayoutEnum(enum.IntEnum):
    SeparationSheet            = 0
    OneFieldLayout             = 1
    TwoGraphLayout             = 7
    Disclaimer                 = 5
    EndSlide                   = 6
    SensorListAndLegend        = 9
    FourGraphLayout            = 12
    FourGraphLayoutCaptions    = 11
    ThreeGraphLayoutCaptions   = 15
    OneGraphLayout             = 16
    OnlyTableLayout            = 17
    TwoGraphTxAntDiagLayout    = 13
    TwoImagesPlaceHolderLayout = 13
    StatsLimitsGroupsLayout    = 18
    TwoGraphLayoutCurves       = 19
    OneGraphLayoutsetcurves    = 20
    OneGraphLayoutstats        = 21

class StatsLimitsGroupsLayoutEnum(enum.IntEnum):
    Title    = 0
    Image1   = 1
    Text     = 2
    Image2   = 3
    Caption1 = 4
    Caption2 = 5
    Table1   = 6
    Table2   = 7

# Mapping of one field slide layout elements
class SeparationSheetEnum(enum.IntEnum):
    Title = 0
    Text  = 1

# Mapping all on one slide layout elements
class AllOnOneSlideLayoutEnum(enum.IntEnum):
    Title   = 0
    Text    = 1
    Image   = 2
    Caption = 3

# Mapping of one field slide layout elements
class OneFieldLayoutEnum(enum.IntEnum):
    TitleText = 0
    Text      = 1
    Title     = 2 

# Single legend slide layout elements mapping
class SingleLegendLayoutEnum(enum.IntEnum):
    Title     = 0
    LegendImg = 1

# Two graph layout slide elements mapping
class TwoGraphLayoutEnum(enum.IntEnum):
    Title        = 0
    PrimText     = 2
    PrimGraphImg = 1
    SecGraphImg  = 3


# Two graph layout slide elements mapping
class TwoGraphLayoutCurvesEnum(enum.IntEnum):
    Title        = 0
    PrimText     = 2
    SecGraphCap  = 5
    PrimGraphImg = 1
    PrimGraphCap = 4
    SecGraphImg  = 3
    # SecGraphCap=6
# Three graph layout slide elements mapping
class ThreeGraphLayoutCaptionsEnum(enum.IntEnum):
    Image1 = 0
    Image2 = 6
    Image3 = 7

    Title = 1
    Text  = 2

    Image1Caption = 3
    Image2Caption = 4
    Image3Caption = 5

# Four graph layout slide elements mapping
class FourGraphLayoutEnum(enum.IntEnum):
    Image1 = 2
    Image2 = 3
    Image3 = 4
    Image4 = 5

    Title         = 0
    Text          = 1
    Image1Caption = 6
    Image2Caption = 7
    Image3Caption = 8
    Image4Caption = 9

# # Four graph layout slide elements mapping
# class FourGraphLayoutCaptionsEnum(enum.IntEnum):
#     Image1 = 0
#     Image2 = 2
#     Image3 = 4
#     Image4 = 5
#     Title=1
#     Text=3


# Two graph slide for TxAnt diagrams
class TwoGraphTxAntDiagLayoutEnum(enum.IntEnum):
    Title         = 0
    Image1        = 1
    Text          = 2
    Image1Caption = 3
    Image2Caption = 4
    Image2        = 5

# Two graph slide for TxAnt diagrams
class TwoImagesPlaceholderLayoutEnum(enum.IntEnum):
    Title         = 0
    Image1        = 1
    Text          = 2
    Image1Caption = 3
    Image2Caption = 4
    Image2        = 5

# Sensor list and legend slide layout elements mapping
class SensorListAndLegendEnum(enum.IntEnum):
    Title           = 0
    Text            = 1
    LegendImg       = 2
    SensorListTable = 3
    LegendCap       = 4

# Sensor list and legend slide layout elements mapping
class OnlyTableEnum(enum.IntEnum):
    Title = 0
    Text  = 1
    Table = 2


# pptx presentation class
class Presentation:
    def __init__(self, starttemp):
        # temp = sys.modules[self.__module__].__file__
        temp = os.path.dirname(sys.argv[0])
        # temp = os.path.split(temp)
        # self.template = os.path.join(temp[0],os.path.normpath(starttemp))
        self.template = os.path.join(temp,os.path.normpath(starttemp))
        self.openStatus = True
        self.saveStatus = True
        self.invalidFileType = False
        self.openStatusString = ''
        self.saveStatusString = ''
        try:
            self.prt = pptx.Presentation(self.template)
        except Exception as e:
            self.openStatusString = 'Opening pptx: ' +str(e)
            self.openStatus = False


    # add a slide function for input slide layout
    def add_slide(self, slideLayout):
        slideLayout.setLayout(self)
        return slideLayout

    # save pptx presentation for input file name
    def save(self, pptxPath):

        # add Disclaimer
        # add end slide
        if 'combine' not in pptxPath:
            self.prt.slides.add_slide(self.prt.slide_layouts[5])
        

        # Save location processing
        # currentDir = os.path.dirname(os.path.abspath(__file__))
        currentDir = os.path.dirname(sys.argv[0])
        savePath   = os.path.join(currentDir, pptxPath)

        try:
            self.prt.save(pptxPath)
        except Exception as e:
            self.saveStatusString = e
            self.saveStatus = False



# Abstract slide layout class
class SlideLayout(ABC):
    @abstractmethod
    def setLayout(self):
        pass
    # Get info about the actual slide elements layout numbering
    def getShapesList(self):
        for shape in self.slide.placeholders:
            print('%d %s' % (shape.placeholder_format.idx, shape.name))

    def getNofGraphs(self):
        pass

# statsLimitsGroupsLayout

class StatsLimitsGroupsLayout(SlideLayout):
    def setLayout(self,Presentation):
        self.slide    = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.StatsLimitsGroupsLayout])

        self.title    = self.slide.shapes[StatsLimitsGroupsLayoutEnum.Title]
        self.text     = self.slide.shapes[StatsLimitsGroupsLayoutEnum.Text]
        self.caption1 = self.slide.shapes[StatsLimitsGroupsLayoutEnum.Caption1]
        self.caption2 = self.slide.shapes[StatsLimitsGroupsLayoutEnum.Caption2]
        self.image1   = self.slide.shapes[StatsLimitsGroupsLayoutEnum.Image1]
        self.image2   = self.slide.shapes[StatsLimitsGroupsLayoutEnum.Image2]
        self.table1   = self.slide.shapes[StatsLimitsGroupsLayoutEnum.Table1]
        self.table2   = self.slide.shapes[StatsLimitsGroupsLayoutEnum.Table2]



# All on one slide layout class
class OneGraphLayout(SlideLayout):
    def setLayout(self,Presentation):
        self.slide   = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.OneGraphLayout])
        self.title   = self.slide.shapes[AllOnOneSlideLayoutEnum.Title]
        self.text    = self.slide.shapes[AllOnOneSlideLayoutEnum.Text]
        self.caption = self.slide.shapes[AllOnOneSlideLayoutEnum.Caption]
        self.image   = self.slide.shapes[AllOnOneSlideLayoutEnum.Image]

    def getNofGraphs(self):
        return 1

# All on one slide layout class setcurves
class OneGraphLayoutsetcurves(SlideLayout):
    def setLayout(self,Presentation):
        self.slide   = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.OneGraphLayoutsetcurves])
        self.title   = self.slide.shapes[AllOnOneSlideLayoutEnum.Title]
        self.text    = self.slide.shapes[AllOnOneSlideLayoutEnum.Text]
        self.caption = self.slide.shapes[AllOnOneSlideLayoutEnum.Caption]
        self.image   = self.slide.shapes[AllOnOneSlideLayoutEnum.Image]

    def getNofGraphs(self):
        return 1

# All on one slide layout class stats
class OneGraphLayoutstats(SlideLayout):
    def setLayout(self,Presentation):
        self.slide   = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.OneGraphLayoutstats])
        self.title   = self.slide.shapes[AllOnOneSlideLayoutEnum.Title]
        self.text    = self.slide.shapes[AllOnOneSlideLayoutEnum.Text]
        self.caption = self.slide.shapes[AllOnOneSlideLayoutEnum.Caption]
        self.image   = self.slide.shapes[AllOnOneSlideLayoutEnum.Image]

    def getNofGraphs(self):
        return 1


# Two-graph layout class
class TwoGraphLayout(SlideLayout):
    def setLayout(self,Presentation):
        self.slide        = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.TwoGraphLayout])
        self.title        = self.slide.shapes[TwoGraphLayoutEnum.Title]
        self.primText     = self.slide.shapes[TwoGraphLayoutEnum.PrimText]
        self.primGraphImg = self.slide.shapes[TwoGraphLayoutEnum.PrimGraphImg]
        self.secGraphImg  = self.slide.shapes[TwoGraphLayoutEnum.SecGraphImg]


# Two-graph layout class Curves
class TwoGraphLayoutCurves(SlideLayout):
    def setLayout(self,Presentation):
        self.slide        = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.TwoGraphLayoutCurves])
        self.title        = self.slide.shapes[TwoGraphLayoutCurvesEnum.Title]
        self.primText     = self.slide.shapes[TwoGraphLayoutCurvesEnum.PrimText]
        self.primGraphImg = self.slide.shapes[TwoGraphLayoutCurvesEnum.PrimGraphImg]
        self.secGraphImg  = self.slide.shapes[TwoGraphLayoutCurvesEnum.SecGraphImg]
        self.primGraphCap = self.slide.shapes[TwoGraphLayoutCurvesEnum.PrimGraphCap]
        self.secGraphCap  = self.slide.shapes[TwoGraphLayoutCurvesEnum.SecGraphCap]

# One-graph layout class
class OneFieldLayout(SlideLayout):
    def setLayout(self,Presentation):
        self.slide     =Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.OneFieldLayout])
        self.title     = self.slide.shapes[OneFieldLayoutEnum.Title]
        self.titletext = self.slide.shapes[OneFieldLayoutEnum.TitleText]
        self.text      = self.slide.shapes[OneFieldLayoutEnum.Text]

# One-graph layout class
class SeparationSheet(SlideLayout):
    def setLayout(self,Presentation):
        self.slide = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.SeparationSheet])
        self.title = self.slide.shapes[SeparationSheetEnum.Title]
        self.text  = self.slide.shapes[SeparationSheetEnum.Text]
      #   self.placeholder = self.slide.shapes[SeparationSheetEnum.Placeholder]


# Sensor-and-list layout class
class SensorListAndLegend(SlideLayout):
    def setLayout(self,Presentation):
        self.slide            = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.SensorListAndLegend])
        self.title            = self.slide.shapes[SensorListAndLegendEnum.Title]
        self.text             = self.slide.shapes[SensorListAndLegendEnum.Text]
        self.legendImg        = self.slide.shapes[SensorListAndLegendEnum.LegendImg]
        self.legendCap        = self.slide.shapes[SensorListAndLegendEnum.LegendCap]
        self.tablePlaceholder = self.slide.shapes[SensorListAndLegendEnum.SensorListTable]

class OnlyTableLayout(SlideLayout):
    def setLayout(self,Presentation):
        self.slide            =Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.OnlyTableLayout])
        self.title            = self.slide.shapes[OnlyTableEnum.Title]
        self.text             = self.slide.shapes[OnlyTableEnum.Text]
        self.tablePlaceholder = self.slide.shapes[OnlyTableEnum.Table]

# Four-graph layout class
class FourGraphLayout(SlideLayout):
    def setLayout(self,Presentation):
        self.imageList    = []
        self.captionsList = []
        self.slide   = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.FourGraphLayout])
        self.title   = self.slide.shapes[FourGraphLayoutEnum.Title]
        self.text    = self.slide.shapes[FourGraphLayoutEnum.Text]
        self.img1    = self.slide.shapes[FourGraphLayoutEnum.Image1]
        self.img2    = self.slide.shapes[FourGraphLayoutEnum.Image2]
        self.img3    = self.slide.shapes[FourGraphLayoutEnum.Image3]
        self.img4    = self.slide.shapes[FourGraphLayoutEnum.Image4]
        self.img1Cap = self.slide.shapes[FourGraphLayoutEnum.Image1Caption]
        self.img2Cap = self.slide.shapes[FourGraphLayoutEnum.Image2Caption]
        self.img3Cap = self.slide.shapes[FourGraphLayoutEnum.Image3Caption]
        self.img4Cap = self.slide.shapes[FourGraphLayoutEnum.Image4Caption]
        self.getImageList()
        self.getCaptionsList()

    def getImageList(self):
        self.imageList.append(self.img1)
        self.imageList.append(self.img2)
        self.imageList.append(self.img3)
        self.imageList.append(self.img4)

    def getCaptionsList(self):
        self.captionsList.append(self.img1Cap)
        self.captionsList.append(self.img2Cap)
        self.captionsList.append(self.img3Cap)
        self.captionsList.append(self.img4Cap)


    def getNofGraphs(self):
        return 4

# Three-graph layout class
class ThreeGraphLayout(SlideLayout):
    def setLayout(self,Presentation):
        self.imageList    = []
        self.captionsList = []
        self.slide   = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.ThreeGraphLayoutCaptions])
        self.title   = self.slide.shapes[ThreeGraphLayoutCaptionsEnum.Title]
        self.text    = self.slide.shapes[ThreeGraphLayoutCaptionsEnum.Text]
        self.img1    = self.slide.shapes[ThreeGraphLayoutCaptionsEnum.Image1]
        self.img2    = self.slide.shapes[ThreeGraphLayoutCaptionsEnum.Image2]
        self.img3    = self.slide.shapes[ThreeGraphLayoutCaptionsEnum.Image3]
        self.img1Cap = self.slide.shapes[ThreeGraphLayoutCaptionsEnum.Image1Caption]
        self.img2Cap = self.slide.shapes[ThreeGraphLayoutCaptionsEnum.Image2Caption]
        self.img3Cap = self.slide.shapes[ThreeGraphLayoutCaptionsEnum.Image3Caption]

        self.getImageList()
        self.getCaptionsList()

    def getImageList(self):
        self.imageList.append(self.img1)
        self.imageList.append(self.img2)
        self.imageList.append(self.img3)

    def getCaptionsList(self):
        self.captionsList.append(self.img1Cap)
        self.captionsList.append(self.img2Cap)
        self.captionsList.append(self.img3Cap)

    def getNofGraphs(self):
        return 3

# Two-graph layout class
class TwoGraphTxAntDiagLayout(SlideLayout):
    def setLayout(self,Presentation):
        self.imageList    = []
        self.captionsList = []
        self.slide     = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.TwoGraphTxAntDiagLayout])
        self.title     = self.slide.shapes[TwoGraphTxAntDiagLayoutEnum.Title]
        self.text      = self.slide.shapes[TwoGraphTxAntDiagLayoutEnum.Text]
        self.image1    = self.slide.shapes[TwoGraphTxAntDiagLayoutEnum.Image1]
        self.image2    = self.slide.shapes[TwoGraphTxAntDiagLayoutEnum.Image2]
        self.image1Cap = self.slide.shapes[TwoGraphTxAntDiagLayoutEnum.Image1Caption]
        self.image2Cap = self.slide.shapes[TwoGraphTxAntDiagLayoutEnum.Image2Caption]

        self.getImageList()
        self.getCaptionsList()

    def getImageList(self):
        self.imageList.append(self.image1)
        self.imageList.append(self.image2)


    def getCaptionsList(self):
        self.captionsList.append(self.image1Cap)
        self.captionsList.append(self.image2Cap)

    def getNofGraphs(self):
        return 2

# Two-graph layout class
class TwoImagesPlaceHolder(SlideLayout):
    def setLayout(self,Presentation):
        self.imageList    = []
        self.captionsList = []
        self.slide     = Presentation.prt.slides.add_slide(Presentation.prt.slide_layouts[SlideLayoutEnum.TwoImagesPlaceHolderLayout])
        self.title     = self.slide.shapes[TwoImagesPlaceholderLayoutEnum.Title]
        self.text      = self.slide.shapes[TwoImagesPlaceholderLayoutEnum.Text]
        self.image1    = self.slide.shapes[TwoImagesPlaceholderLayoutEnum.Image1]
        self.image2    = self.slide.shapes[TwoImagesPlaceholderLayoutEnum.Image2]
        self.image1Cap = self.slide.shapes[TwoImagesPlaceholderLayoutEnum.Image1Caption]
        self.image2Cap = self.slide.shapes[TwoImagesPlaceholderLayoutEnum.Image2Caption]

        self.getImageList()
        self.getCaptionsList()

    def getImageList(self):
        self.imageList.append(self.image1)
        self.imageList.append(self.image2)


    def getCaptionsList(self):
        self.captionsList.append(self.image1Cap)
        self.captionsList.append(self.image2Cap)

    def getNofGraphs(self):
        return 2



# Graph list to slide mapper class, for all Rx channels together drawn graphs.
class TwoGraphConfig:
    def __init__(self, twoGraphConfig, graphAndDiagramList, diagramNr, graphList):
        self.primGraph    = 0
        self.secGraph     = 0
        self.onlyOneGraph = 0
        self.setPrimGraph(twoGraphConfig,graphAndDiagramList,diagramNr)
        self.setSecGraph(twoGraphConfig,graphAndDiagramList,diagramNr)
        self.getOnlyOneGraph(twoGraphConfig, graphList)

    # Get only one graph
    def getOnlyOneGraph(self, twoGraphConfig, graphList):
        if twoGraphConfig['primGraph'] in graphList and twoGraphConfig['secGraph'] not in graphList:
            self.onlyOneGraph = self.primGraph
        elif twoGraphConfig['primGraph'] in graphList and twoGraphConfig['secGraph']  in graphList:
            self.onlyOneGraph = 0
        elif twoGraphConfig['primGraph'] not in graphList and twoGraphConfig['secGraph'] in graphList:
            self.onlyOneGraph = self.secGraph
        else:
            self.onlyOneGraph =0

    # Set the data for the primary graphs in the two-graph layout, per one diagram nr.
    def setPrimGraph(self, twoGraphConfig, graphAndDiagramList, diagramNr):
        gdListShape = np.shape(graphAndDiagramList)
        nofGraphs   = gdListShape[0]
        nofDiagrams = gdListShape[1]
        for graphIdx in range(nofGraphs):
            for diagramIdx in range(nofDiagrams):
                if graphAndDiagramList[graphIdx][diagramIdx].graphTitle == twoGraphConfig["primGraph"] and diagramNr == graphAndDiagramList[graphIdx][diagramIdx].diagramNr:
                    self.primGraph = graphAndDiagramList[graphIdx][diagramIdx]

    # Set the data for the secondary graphs in the two-graph layout, per one diagram nr.
    def setSecGraph(self, twoGraphConfig, graphAndDiagramList, diagramNr):
        gdListShape = np.shape(graphAndDiagramList)
        nofGraphs   = gdListShape[0]
        nofDiagrams = gdListShape[1]
        for graphIdx in range(nofGraphs):
            for diagramIdx in range(nofDiagrams):
                if graphAndDiagramList[graphIdx][diagramIdx].graphTitle == twoGraphConfig["secGraph"] and diagramNr == graphAndDiagramList[graphIdx][diagramIdx].diagramNr:
                    self.secGraph = graphAndDiagramList[graphIdx][diagramIdx]

# Graph list to slide mapper class, for all Rx channels separate drawn graphs.
class TwoGraphConfigRx(TwoGraphLayout):
    def __init__(self, twoGraphConfig, graphDiagramRxChList, diagramNr, graphList):
        self.primGraphList    = []
        self.secGraphList     = []
        self.onlyOneGraphList = 0
        self.setPrimGraph(twoGraphConfig, graphDiagramRxChList, diagramNr)
        self.setSecGraph(twoGraphConfig, graphDiagramRxChList, diagramNr)
        self.getOnlyOneGraph(twoGraphConfig, graphList)

    # Get only one graph
    def getOnlyOneGraph(self, twoGraphConfig, graphList):
        if twoGraphConfig['primGraph'] in graphList and twoGraphConfig['secGraph'] not in graphList:
            self.onlyOneGraphList = self.primGraphList
        elif twoGraphConfig['primGraph'] in graphList and twoGraphConfig['secGraph'] in graphList:
            self.onlyOneGraphList = 0
        elif twoGraphConfig['primGraph'] not in graphList and twoGraphConfig['secGraph'] in graphList:
            self.onlyOneGraphList = self.secGraphList
        else:
            self.onlyOneGraphList = 0

    # Set the list for the primary graphs in the two-graph layout, per one diagram nr.
    def setPrimGraph(self, twoGraphConfig, graphDiagramRxChList, diagramNr):
        gdListShape = np.shape(graphDiagramRxChList)
        nofGraphs   = gdListShape[0]
        nofDiagrams = gdListShape[1]
        for graphIdx in range(nofGraphs):
            for diagramIdx in range(nofDiagrams):
                rxChList = graphDiagramRxChList[graphIdx][diagramIdx]
                for rxCh in range(len(rxChList)):
                    if rxChList[rxCh].graphTitle == twoGraphConfig["primGraph"] and diagramNr == rxChList[rxCh].diagramNr:
                        self.primGraphList.append(rxChList[rxCh])

    # Set the list for the secondary graphs in the two-graph layout, per one diagram nr.
    def setSecGraph(self, twoGraphConfig, graphDiagramRxChList, diagramNr):
        gdListShape = np.shape(graphDiagramRxChList)
        nofGraphs   = gdListShape[0]
        nofDiagrams = gdListShape[1]
        for graphIdx in range(nofGraphs):
            for diagramIdx in range(nofDiagrams):
                rxChList = graphDiagramRxChList[graphIdx][diagramIdx]
                for rxCh in range(len(rxChList)):
                    if rxChList[rxCh].graphTitle == twoGraphConfig["secGraph"] and diagramNr == rxChList[rxCh].diagramNr:
                        self.secGraphList.append(rxChList[rxCh])

class FourGraphConfigRx(TwoGraphLayout):

    def __init__(self, fourGraphConfig, graphDiagramRxChList, diagramNr, graphList):
        self.graphList = []
        self.setGraphList(fourGraphConfig, graphDiagramRxChList, diagramNr)

    def setGraphList(self, fourGraphConfig, graphDiagramRxChList, diagramNr):
        gdListShape = np.shape(graphDiagramRxChList)
        nofGraphs   = gdListShape[0]
        nofDiagrams = gdListShape[1]
        for graphIdx in range(nofGraphs):
            for diagramIdx in range(nofDiagrams):
                rxChList = graphDiagramRxChList[graphIdx][diagramIdx]
                for rxCh in range(len(rxChList)):
                    if rxChList[rxCh].graphTitle == fourGraphConfig["graphType"] and diagramNr == rxChList[
                        rxCh].diagramNr:
                        self.graphList.append(rxChList[rxCh])

class ThreeGraphConfigRx(TwoGraphLayout):

    def __init__(self, threeGraphConfig, graphDiagramRxChList, diagramNr, graphList):
        self.graphList = []
        self.setGraphList(threeGraphConfig, graphDiagramRxChList, diagramNr)

    def setGraphList(self, threeGraphConfig, graphDiagramRxChList, diagramNr):
        gdListShape = np.shape(graphDiagramRxChList)
        nofGraphs   = gdListShape[0]
        nofDiagrams = gdListShape[1]
        for graphIdx in range(nofGraphs):
            for diagramIdx in range(nofDiagrams):
                rxChList = graphDiagramRxChList[graphIdx][diagramIdx]
                for rxCh in range(len(rxChList)):
                    if rxChList[rxCh].graphTitle == threeGraphConfig["graphType"] and diagramNr == rxChList[
                        rxCh].diagramNr:
                        self.graphList.append(rxChList[rxCh])



import os
import parsePTU2data as pd
import enum
import logconfig as log


myLogger = log.mainLoadDataLogger.logger


# absPath=r'C:\Arbeit\ProtokollBsp\UMRR-11.00.04-0002B4ED'
absPath=r'C:\Arbeit\ProtokollBsp'

# Search path meta data
class SearchPath(enum.Enum):
    Invalid=0
    Folder=1
    Protocol=2
    File=3
    RootFolder=4
class AbsPathType(enum.Enum):
    Path=0
    List=1
    Tuple=2

# Class containing the loaded data, including the parsed PTU2 files list, error status data, and meta data
class LoadData:
    def __init__(self,absPath,useSensorMuster,sensorMusterList):
        self.searchPath = SearchPath.Invalid
        self.parseDataList = []
        self.sensorMusterList=sensorMusterList
        self.useSensorMuster=useSensorMuster
        self.errorString=''
        self.errorStringList=[]
        self.invalidPath=0
        self.invalidPathString = '\n Invalid path given! Use files with "_Protocol.text", "_PTU2.txt" endings, and make sure the path is valid.'
        self.counter = 0
        self.validFilesCounter=0
        self.loadData(absPath)
        self.parseLogging()

    # Logging for the parsing errors
    def parseLogging(self):
        errorString=''
        if self.counter >0:
            for i in range(len(self.errorStringList)):
                errorString+=self.errorStringList[i]
            myLogger.error('Parsing errors: '+errorString)

   # Formating function for the parsing errors
    def addParseDataError(self,parseData):
        logStr = ""
        for errStrIdx in range(len(parseData.errorList)):
            logStr = logStr + "\n" + parseData.errorList[errStrIdx]
        return logStr


    def proceessPath(self,absPath):
        ptu2PathEnd = "_PTU2.txt"
        protocolPathEnd = '_Protocol.txt'

        pathParts = os.path.split(absPath)

        if pathParts[1].endswith(ptu2PathEnd):
            if 'Diff' not in pathParts[1] and 'NotOk' not in pathParts[1]:
                parseData = pd.PTU2ParseData()
                parseData.parseFile(absPath,self.useSensorMuster,self.sensorMusterList)
                self.searchPath = SearchPath.File
                if parseData.bDataValid == False:
                    self.counter += 1
                    self.errorString = self.addParseDataError(parseData)
                    self.errorStringList.append(self.errorString)
                    # myLogger.error(self.errorString)
                else:
                    self.parseDataList.append(parseData)
                    self.validFilesCounter += 1

        elif pathParts[1].endswith(protocolPathEnd):
            pathError = 0
            lookFurther = 0
            self.searchPath = SearchPath.Protocol
            for item in os.listdir(pathParts[0]):
                if item.endswith(pathParts[1]):
                    lookFurther = 1

            if lookFurther == 1:
                for item in os.listdir(pathParts[0]):
                    if item.endswith(ptu2PathEnd):
                        if 'Diff' not in item and 'NotOk' not in item:
                            # Set path to current file
                            # pathToFile = pathParts[0] + "\\" + item
                            pathToFile = os.path.join(pathParts[0], item)
                            parseData = pd.PTU2ParseData()
                            # obj=parseFile(pathToFile,obj)
                            parseData.parseFile(pathToFile,self.useSensorMuster,self.sensorMusterList)
                            if parseData.bDataValid == False:
                                self.counter += 1
                                self.errorString = self.addParseDataError(parseData)
                                self.errorStringList.append(self.errorString)
                                # myLogger.error(self.errorString)
                            else:
                                self.validFilesCounter +=1
                                self.parseDataList.append(parseData)
            else:
                pathError = 1

            if pathError == 1:
                self.invalidPath = 1
                self.invalidPathString = absPath + self.invalidPathString
                # myLogger.error(self.errorString)
        else:

            if os.path.isdir(absPath):
                self.searchPath = SearchPath.Folder
                containsPtu2 = 0

                for dirName, subdirList, fileList in os.walk(absPath):
                    subdirList[:] = [d for d in subdirList if 'NotOk' not in d]
                    # Sort fileList to ensure consistent order (preserves alphabetical/chronological order)
                    fileList.sort()
                    for fileName in fileList:
                        if fileName.endswith(ptu2PathEnd):
                            if 'Diff' not in fileName and 'NotOk' not in fileName:

                                containsPtu2 = 1
                                # Set path to current file
                                # pathToFile = subPath + "\\" + file

                                pathToFile = os.path.join(dirName, fileName)

                                parseData = pd.PTU2ParseData()
                                parseData.parseFile(pathToFile,self.useSensorMuster,self.sensorMusterList)
                                if parseData.bDataValid == False:
                                    self.counter += 1
                                    self.errorString = self.addParseDataError(parseData)
                                    self.errorStringList.append(self.errorString)
                                    # myLogger.error(self.errorString)
                                else:
                                    self.validFilesCounter += 1
                                    self.parseDataList.append(parseData)

                if containsPtu2 == 0:
                    self.invalidPath = 1
                    self.invalidPathString = absPath + self.invalidPathString
                    # myLogger.error(self.errorString)
            else:
                self.invalidPath = 1
                self.invalidPathString = absPath + self.invalidPathString
            # myLogger.error(self.errorString)

    # Load parsed data for a specific search path
    def loadData(self,absPath):

        absPathType=AbsPathType.Path

        if isinstance(absPath, tuple):
            absPathType=AbsPathType.Tuple

        elif isinstance(absPath, list):
            absPathType=AbsPathType.List

        if absPathType==AbsPathType.Path:
            self.proceessPath(absPath)

        elif absPathType == AbsPathType.List:
              #absPath=str(absPath[0])
            absPathLen = len(absPath)
            for fileIdx in range (absPathLen):
                absPathInstance = str(absPath[fileIdx])
                self.proceessPath(absPathInstance)


        elif absPathType==AbsPathType.Tuple:
            absPathLen = len(absPath)
            for fileIdx in range (absPathLen):
                absPathInstance = str(absPath[fileIdx])
                self.proceessPath(absPathInstance)



# l=LoadData(absPath)
# a=getNofDiagrams(l)
# sensorList=getSensorList(l)
# y=6



from shared import safeparse as sp
from logconfig import logAdapter_PP as log
import yaml
import enum
import codecs
import copy
import re

# log.error('Error')
# log.warning('Warning')
# log.info('info')

class ProtocolType(enum.Enum):
    BurnIn ='BurnIn'
    Calibration = 'Calibration'

class MeasResultEnum(enum.Enum):
    NotUsed = -2
    NotMeasured = -1
    MeasNotOK = 0
    MeasOk = 1
    Invalid = 2

class ProtocolCategoriesEnum(enum.Enum):
    ProtocolHead = 0
    MeasurementLimits = 1
    Measurements = 2
    AbbortCriterions = 3
    MeasurementDurations = 4
    MeasSettings = 5

class ProtocolCategoryNameEnum(enum.Enum):
    MeasurementLimits = 'MeasurmentLimit'
    AbbortCriterions  = 'AbbortCriterions'


class ErrorStatusEnum(enum.Enum):
    Ok = 0
    Error = 1
    Warning = 2
#
# class LimitCategory():
#     def __init__(self):
#         self.limitsList=[]


class ProtocolHeadEnum(enum.Enum):
    PtuOffsets_Az ='PtuOffset: AZ[°]:'
    PtuOffsets_El ='PtuOffset: EL[°]:'
    CalibrationDspSoftwareInfo = 'Calibration DSP softwareinfo:'
    CalibrationDspSoftwareCompileInfo = 'Calibration DSP software Compile Info:'
    ModularSoftware = 'Modular Software:'
    CalStart = 'CalStart:'
    CalEnd   = 'CalEnd:'
    MeasDuration   = 'Duration[s]: '
    MeasDurationV2 = 'DurationV2[s]:'
    RFSerial = 'RF-Serial:'
    BurnInSpeed = 'Burn16a_BurnInMeasSpeedList[km/h]'
    BurnInAzi   = 'Burn16c_BurnInMeasPtuPosAzimuthList[°]'
    BurnInEle   = 'Burn16d_BurnInPtuPosElevationList[°]'


class ProtocolHeadElement:
    def __init__(self,name):
        self.name   = name
        self.lineNr = 0
        self.nameFound = 0
        self.errorStatus = ErrorStatusEnum.Ok
        self.valuesList  = []

class ProtocolHeadElementValue:
    def __init__(self,name):
        self.name  = name
        self.value = ''

        self.nameFound  = 0
        self.valueFound = 0

class MeasurementEnum(enum.Enum):
    TxAntRawFrequencyTable_Measpoints = 0
    TxAntPowerCorrCurve = 2
    TxAntCtapRawFreqSelectSearch = 1

class MeasurementDurationsEnum(enum.Enum):
    TxAnt = 'TxAnt[1]'
    Frequency_MHz = 'Freq[MHz]'
    Level_dB = 'Level[dB]'
    Level_Powermeter_dB = 'Level-Powermeter[dB]'

class MeasurementDuration():
    def __init__(self):
        self.name = ''
        self.nameFound = 0

        self.value = 0
        self.valueFound = 0

        self.start = ''
        self.end   = ''
        self.startFound = 0
        self.endFound = 0

        self.errorStatus = ErrorStatusEnum.Ok

class MeasSetting():
    def __init__(self):
        self.name = ''
        self.nameFound = 0

        self.value = ''
        self.valueFound = 0

        self.comment = ''
        self.commentFound = 0

        self.errorStatus = ErrorStatusEnum.Ok

NOF_TX_ANT = 3
MeasurementType3aDict = dict()
MeasurementType3bDict = dict()
MeasurementType3eDict = dict()
TxAntPowerCorrCurveDict   = dict()
BasicMeasurementParamDict = dict()

for txAnt in range (NOF_TX_ANT):
    extraString = ''
    strAddString  = 'TxAnt' + str(txAnt)
    strAddString2 = 'Tx' + str(txAnt)

    basicString = '(3a) ' + strAddString + ' raw frequency table -  measpoints'
    possibleNames  = [ '(3a) ' + extraString + 'raw frequency table -  measpoints' ]
    possibleNames += ['(3a)' + strAddString]
    possibleNames += ['(3a)' + extraString]
    possibleNames += ['(3a' + strAddString2 + ')']
    MeasurementType3aDict[basicString] = possibleNames

    basicString = '(3e) ' + strAddString + ' TxPowerCorrCurve'
    possibleNames  = [ '(3e) ' + strAddString + 'TxPowerCorrCurve']
    possibleNames += ['(3e)' + strAddString]
    possibleNames += ['(3e)' + extraString]
    possibleNames += ['(3e' + strAddString2 + ')']
    MeasurementType3eDict[basicString] = possibleNames

    basicString = '(3b) ' + strAddString + ' ctap raw freq select search'
    possibleNames  = [ '(3b) ' + strAddString + 'ctap raw freq select search']
    possibleNames += ['(3b)' + strAddString]
    possibleNames += ['(3b)' + extraString]
    possibleNames += ['(3b' + strAddString2 + ')']
    MeasurementType3bDict[basicString] = possibleNames

class BasicMeasurementParamEnum(enum.Enum):
    TxAnt = 'TxAnt[1]'
    Frequency_MHz = 'Freq[MHz]'
    Level_dB = 'Level[dB]'
    Level_Powermeter_dB = 'Level-Powermeter[dB]'
    FreqSensorSetting   = 'FreqSensorSetting[MHz]'
    RawFreqSel = 'raw freq sel[1]'
    DAC = 'DAC[dac] '
    TIME_ms  = 'Time[ms] '
    TXPowVal = 'tx_pow_val'
    Number = 'Number [1]'
    CenterFrequency_MHz = 'Centerfreq [Mhz]'
    Span_MHz = 'Span [MHz]'
    LevelMeasSpec_dB = 'Level meas (Spec)[dB]'
    LevelMeasPowermeter_dB = 'Level meas (Powermeter)[dB]'

BasicMeasurementParamDict[BasicMeasurementParamEnum.TxAnt] = [BasicMeasurementParamEnum.TxAnt.value]
BasicMeasurementParamDict[BasicMeasurementParamEnum.Frequency_MHz] = [BasicMeasurementParamEnum.Frequency_MHz.value]
BasicMeasurementParamDict[BasicMeasurementParamEnum.Level_dB] = [BasicMeasurementParamEnum.Level_dB.value]
BasicMeasurementParamDict[BasicMeasurementParamEnum.Level_Powermeter_dB] = [BasicMeasurementParamEnum.Level_Powermeter_dB.value]

BasicMeasurementParamDict[BasicMeasurementParamEnum.FreqSensorSetting] = [BasicMeasurementParamEnum.FreqSensorSetting.value]

BasicMeasurementParamDict[BasicMeasurementParamEnum.RawFreqSel] = [BasicMeasurementParamEnum.RawFreqSel.value]
BasicMeasurementParamDict[BasicMeasurementParamEnum.DAC] = [BasicMeasurementParamEnum.DAC.value]

BasicMeasurementParamDict[BasicMeasurementParamEnum.TIME_ms]  = [BasicMeasurementParamEnum.TIME_ms.value]
BasicMeasurementParamDict[BasicMeasurementParamEnum.TXPowVal] = [BasicMeasurementParamEnum.TXPowVal.value,'tx_power_val[1]']

BasicMeasurementParamDict[BasicMeasurementParamEnum.Number] = [BasicMeasurementParamEnum.Number.value]
BasicMeasurementParamDict[BasicMeasurementParamEnum.CenterFrequency_MHz] = [BasicMeasurementParamEnum.CenterFrequency_MHz.value]
BasicMeasurementParamDict[BasicMeasurementParamEnum.Span_MHz] = [BasicMeasurementParamEnum.Span_MHz.value]
BasicMeasurementParamDict[BasicMeasurementParamEnum.LevelMeasSpec_dB] = [BasicMeasurementParamEnum.LevelMeasSpec_dB.value]
BasicMeasurementParamDict[BasicMeasurementParamEnum.LevelMeasPowermeter_dB] = [BasicMeasurementParamEnum.LevelMeasPowermeter_dB.value]

BasicMeasurementParamListBasicVariant = [BasicMeasurementParamEnum.TxAnt,BasicMeasurementParamEnum.Frequency_MHz,BasicMeasurementParamEnum.Level_dB,BasicMeasurementParamEnum.Level_Powermeter_dB,
                                       BasicMeasurementParamEnum.FreqSensorSetting,
                                       BasicMeasurementParamEnum.TIME_ms, BasicMeasurementParamEnum.TXPowVal,BasicMeasurementParamEnum.Number,
                                       BasicMeasurementParamEnum.CenterFrequency_MHz,BasicMeasurementParamEnum.Span_MHz, BasicMeasurementParamEnum.LevelMeasSpec_dB,BasicMeasurementParamEnum.LevelMeasPowermeter_dB]

BasicMeasurementParamListExtendedVariant = [BasicMeasurementParamEnum.TxAnt,BasicMeasurementParamEnum.Frequency_MHz,BasicMeasurementParamEnum.Level_dB,BasicMeasurementParamEnum.Level_Powermeter_dB,
                                       BasicMeasurementParamEnum.RawFreqSel,BasicMeasurementParamEnum.DAC,
                                       BasicMeasurementParamEnum.TIME_ms, BasicMeasurementParamEnum.TXPowVal,BasicMeasurementParamEnum.Number,
                                       BasicMeasurementParamEnum.CenterFrequency_MHz,BasicMeasurementParamEnum.Span_MHz, BasicMeasurementParamEnum.LevelMeasSpec_dB,BasicMeasurementParamEnum.LevelMeasPowermeter_dB]
class BasicMeasurementParamSearchEnum(enum.Enum):
    RightAfterMeasTypeTitle = 0
    AfterOtherParams = 1
    NotAtAll = 2
    AfterOtherParamsNoHeader = 3

class TxAntPowerCorrCurveEnum(enum.Enum):
    PowermeterLevelValuesUsed   = 'PowermeterLevelValuesUsed:'
    PowermeterLevelRecalcActive = 'PowermeterLevelRecalcActive:'
    PowermeterLevelCorrMethod   = 'PowermeterLevelCorrMethod:'

TxAntPowerCorrCurvePLVU_PossibleNames = [TxAntPowerCorrCurveEnum.PowermeterLevelValuesUsed.value,'PLVU:']
TxAntPowerCorrCurveDict[TxAntPowerCorrCurveEnum.PowermeterLevelValuesUsed] = TxAntPowerCorrCurvePLVU_PossibleNames

TxAntPowerCorrCurvePLRA_PossibleNames = [TxAntPowerCorrCurveEnum.PowermeterLevelRecalcActive.value,'PLRA:']
TxAntPowerCorrCurveDict[TxAntPowerCorrCurveEnum.PowermeterLevelRecalcActive] = TxAntPowerCorrCurvePLRA_PossibleNames

TxAntPowerCorrCurvePLCM_PossibleNames = [TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod.value,'PLCM:']
TxAntPowerCorrCurveDict[TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod] = TxAntPowerCorrCurvePLCM_PossibleNames

class MeasurementType():
    def __init__(self,name,possibleNames,type):
        self.name = name
        self.nameFound = 0
        self.parameterMeasurementValueDict = dict()
        self.errorStatus = ErrorStatusEnum.Ok
        self.bassicParamNamesFound = 0
        self.noHeaderProcessed = 0
        self.type = type
        self.possibleNames = possibleNames

        for basicString, possibleNames in MeasurementType3aDict.items():
            if self.name==basicString or self.name in possibleNames:
                for parameter in BasicMeasurementParamEnum:
                    for  basicMeasurementParaBasicName,  basicMeasurementParaPossibleNames in BasicMeasurementParamDict.items():
                        if parameter==basicMeasurementParaBasicName:

                            self.parameterMeasurementValueDict[parameter] = MeasurementValue(parameter,basicMeasurementParaPossibleNames)

        for basicString, possibleNames in MeasurementType3bDict.items():
            if self.name==basicString or self.name in possibleNames:
                for parameter in BasicMeasurementParamEnum:
                    for  basicMeasurementParaBasicName,  basicMeasurementParaPossibleNames in BasicMeasurementParamDict.items():
                        if parameter==basicMeasurementParaBasicName:

                            self.parameterMeasurementValueDict[parameter] = MeasurementValue(parameter,basicMeasurementParaPossibleNames)

        for basicString, possibleNames in MeasurementType3eDict.items():
            if self.name==basicString or self.name in possibleNames:
                for parameter in BasicMeasurementParamEnum:
                    for  basicMeasurementParaBasicName,  basicMeasurementParaPossibleNames in BasicMeasurementParamDict.items():
                        if parameter==basicMeasurementParaBasicName:

                            self.parameterMeasurementValueDict[parameter] = MeasurementValue(parameter,basicMeasurementParaPossibleNames)

        if self.type==MeasurementEnum.TxAntPowerCorrCurve:
            for parameter in TxAntPowerCorrCurveEnum:
                for txAntPowerCorrCurveBasicName, txAntPowerCorrCurvePossibleNames in TxAntPowerCorrCurveDict.items():
                    if parameter==txAntPowerCorrCurveBasicName:
                        self.parameterMeasurementValueDict[parameter]=MeasurementValue(parameter,txAntPowerCorrCurvePossibleNames)


    def searchPatternInPossibleNames(self,pattern,possibleNames):
        status = False
        for item in possibleNames:
            if pattern in item:
                status = True
        return  status

    def searchBasicParamNamesWithoutHeader(self,lineNr):
        order = 0

        for listItem in BasicMeasurementParamListBasicVariant:
            for paramKey, paramValue in self.parameterMeasurementValueDict.items():
                if listItem.value in paramValue.possibleNames:

                    paramValue.nameFound = 1
                    paramValue.lineNr = lineNr
                    paramValue.order  = order

                    self.bassicParamNamesFound = 1
            order += 1


        for paramKey, paramValue in self.parameterMeasurementValueDict.items():
            if paramValue.nameFound!=1:
                if paramValue.name.value==BasicMeasurementParamEnum.TxAnt.value or paramValue.name.value==BasicMeasurementParamEnum.Frequency_MHz.value or paramValue.name.value==BasicMeasurementParamEnum.Level_dB.value:
                    log.error('XMeasurements, measurements type: ' + self.name + ', parameter name not found: ' + paramKey.value + ', line ' + str(lineNr))
                    paramValue.errorStatus = ErrorStatusEnum.Error
                else:
                    log.warning('XMeasurements, measurements type: ' + self.name + ', parameter name not found: ' + paramKey.value + ', line ' + str(lineNr))
                    paramValue.errorStatus = ErrorStatusEnum.Ok
    #
    # def checkPossibleNames(self, name):
    #     for listItem in BasicMeasurementParamDict.items():
    #

    def searchBasicParamNames(self,line,lineNr):

        lineList = line.split(';')

        order = 0
        for lineItem in lineList:
            for paramKey, paramValue in self.parameterMeasurementValueDict.items():
                # if paramKey.value==lineItem:
                if lineItem in paramValue.possibleNames:

                    paramValue.nameFound = 1
                    paramValue.lineNr = lineNr
                    paramValue.order = order

                    self.bassicParamNamesFound = 1
            order += 1

        for paramKey, paramValue in self.parameterMeasurementValueDict.items():
            if paramValue.nameFound!=1:
                if paramValue.name.value ==BasicMeasurementParamEnum.TxAnt.value or paramValue.name.value ==BasicMeasurementParamEnum.Frequency_MHz.value or paramValue.name.value== BasicMeasurementParamEnum.Level_dB.value:
                    log.error('Measurements, measurements type: ' + self.name + ', parameter name not found: ' + paramKey.value + ', line ' + str(lineNr))
                    paramValue.errorStatus = ErrorStatusEnum.Error
                else:
                    log.warning('Measurements, measurements type: ' + self.name+ ', parameter name not found: ' + paramKey.value + ', line ' + str(lineNr))
                    paramValue.errorStatus = ErrorStatusEnum.Ok

    def parseBasicParamNames(self,lineList,lineNr):

        for paramKey, paramValue in self.parameterMeasurementValueDict.items():
            for lineIdx in range(len(lineList)):
                if lineIdx==paramValue.order:

                    if paramValue.name==BasicMeasurementParamEnum.TxAnt:
                        if sp.checkInt(lineList[lineIdx]):
                            paramValue.values.append(int(lineList[lineIdx]))
                        else:
                            paramValue.valuesFound = 0
                            paramValue.errorStatus = ErrorStatusEnum.Error
                            log.warning('Measurements, parsing to int failed, measurements type: ' + self.name + ', parameter: ' + paramKey.value + ', line ' + str(lineNr))
                    else:
                        if sp.checkDecimal(lineList[lineIdx]):
                            paramValue.valuesFound = 1
                            paramValue.values.append(float(lineList[lineIdx]))
                        else:
                            paramValue.valuesFound = 0
                            paramValue.errorStatus = ErrorStatusEnum.Error
                            log.warning('Measurements, parsing to float failed, measurements type: ' + self.name + ', parameter: ' + paramKey.value + ', line ' + str(lineNr))

class MeasurementValue():
    def __init__(self,name,possibleNames = []):
        self.name = name
        self.nameFound = 0

        self.actualName = ''

        self.possibleNames = []
        self.possibleNames = possibleNames

        self.lineNr = 0
        self.order  = -1

        self.values = []
        self.valuesFound = 0

        self.errorStatus = ErrorStatusEnum.Ok

class Limit():
    def __init__(self):
        self.name  = ''
        self.value = 0.0
        self.min = 0.0
        self.max = 0.0
        self.measResult = MeasResultEnum.Invalid
        self.comment = ''

        self.possibleNamesList = []

        self.category = ''
        self.catFound = 0

        self.errorStatus = ErrorStatusEnum.Ok

        self.limitFound = 0
        self.minFound   = 0
        self.maxFound   = 0
        self.valueFound = 0
        self.measResultStringFound = 0
        self.measResultFound = 0
        self.commentFound = 0




class AbbortCriterion():
    def __init__(self):
        self.id = 0
        self.name = ''
        self.values = []
        self.statusCode = ''

        self.errorStatus = ErrorStatusEnum.Ok

        self.valuesFound = 0
        self.statusCodeFound = 0
        self.numberFound = 0
        self.nameFound = 0

class AbbortCriterions():
    def __init__(self):
        self.abbortCriterionList = []

class MeasurementLimit():
    def __init__(self):
        self.limitsList = []

class ProtocolParseInput():
    def __init__(self):
        self.measurementLimit = MeasurementLimit()
        self.abbortCriterions = AbbortCriterions()

class ParseProtocols():
    def __init__(self,protocolType):
        # log.error('Error')
        # log.warning('Warning')
        # log.info('info')
        self.rawParsedMeasurements = dict()

        for basicString,possibleNames in MeasurementType3aDict.items():
            self.rawParsedMeasurements[basicString] = MeasurementType(basicString,possibleNames,MeasurementEnum.TxAntRawFrequencyTable_Measpoints)

        for basicString,possibleNames in MeasurementType3bDict.items():
            self.rawParsedMeasurements[basicString] = MeasurementType(basicString,possibleNames,MeasurementEnum.TxAntCtapRawFreqSelectSearch)

        for basicString,possibleNames in MeasurementType3eDict.items():
            self.rawParsedMeasurements[basicString] = MeasurementType(basicString,possibleNames, MeasurementEnum.TxAntPowerCorrCurve)

        self.rawParsedMeasurementLimit = MeasurementLimit()
        self.rawParsedAbbortCriterions = AbbortCriterions()
        self.rawParsedProtocolHead = dict()
        self.rawParsedMeasurementDurations = []
        self.rawParsedMeasSettings = []

        for element in ProtocolHeadEnum:
            self.rawParsedProtocolHead[element] = ProtocolHeadElement(element)

        self.protocolParseInputDict = dict()
        # self.rawParsedMeasurementLimitDict = dict()
        self.inputFilteredMeasurementLimitDict = dict()
        self.inputFilteredMeasurementLimit = MeasurementLimit()

        self.extInputMeasurementLimitDict = dict()

        self.protocolType = protocolType
        self.errorList    = []
        self.errorStatus  = ErrorStatusEnum.Ok

    def generateMeasurmentLimitDict(self, measurementLimit, protocolParseInputDict):

        outputDict = dict()

        if len(measurementLimit.limitsList)!=0:

            catList = []
            for limit in measurementLimit.limitsList:
                if limit.catFound==1:
                    catList.append(limit.category)

            tempMeasLimDict = dict()


            catSet   = set(catList)
            catList  = list(catSet)
            if len(catList)!=0:
                for cat in catList:
                    tempLimDict = dict()
                    for limit in measurementLimit.limitsList:
                        if limit.limitFound==1 and limit.catFound and limit.category==cat:

                            limitNameArray = limit.name.split('[')

                            limitName = limitNameArray[0]

                            unit = ''

                            basis  = ''
                            suffix =''

                            if limitName.endswith('rs485') or limitName.endswith('RS485'):
                                basis = limitName
                            else:
                                if len(limitNameArray)!=1:
                                    unit = '[' + limitNameArray[1]

                                match = re.match(r'(.*\D)?(\d*)$', limitName, re.I)

                                res = tuple()
                                if match:
                                    res = match.groups()

                                basis  = res[0]
                                suffix = res[1]


                            if cat in protocolParseInputDict['MeasurmentLimit']:

                                for inputLimitBasis,possibleLimBasisList in  protocolParseInputDict['MeasurmentLimit'][cat].items():

                                    if basis==inputLimitBasis:

                                        tempLimDict[inputLimitBasis] = possibleLimBasisList

                                    elif possibleLimBasisList is not None:
                                        if basis in possibleLimBasisList:
                                            tempLimDict[inputLimitBasis] = possibleLimBasisList
                                            newName = inputLimitBasis + suffix + unit
                                            limit.name = newName

                            else:
                                log.error('Generate MeasurementLimits dict: Limits category not found for inputMeasurmentLimitDict["MeasurmentLimit"], category: '+str(cat))
                                self.errorStatus = ErrorStatusEnum.Error

                    tempMeasLimDict[cat] = tempLimDict
            else:
                log.error('Generate MeasurementLimits dict: No limits category found for MeasurementLimit.')
                self.errorStatus = ErrorStatusEnum.Error

            outputDict = tempMeasLimDict

        else:
            log.error('Generate MeasurementLimits dict: Empty limits list for MeasurementLimit.')
            self.errorStatus = ErrorStatusEnum.Error

        return outputDict, measurementLimit

    def generateMeasurmentLimitDictHeaders(self, measurementLimit, protocolParseInputDict):

        outputDict = dict()
        for headobject,headvalue in measurementLimit.items():
            if len(headvalue.valuesList)!=0:
                for value in headvalue.valuesList:
                    if value.value != '':
                        if 'Burn16' in headobject.value:
                            val = value.value.replace(' ', '')
                            val = val.split('|')
                            for elemIdx in range(len(val)):
                                if sp.checkDecimal(val[elemIdx]):
                                    limitNameSplit = value.name.split('_')
                                    limitNameFilter = limitNameSplit[1].split('[')
                                    outputDict[str(limitNameFilter[0])+str(elemIdx)] = [float(val[elemIdx])]
                        else:
                            outputDict[headobject.value] = [value.value]

            else:
                log.error('Generate MeasurementLimits dict: Empty limits list for MeasurementLimit.')
                self.errorStatus = ErrorStatusEnum.Error

        return outputDict

    def extendInputMeasurementLimitDict(self):

        log.info('Extending '+str(self.protocolType)+' input MeasurementLimit dict.')

        if len(self.rawParsedMeasurementLimit.limitsList)==0:
            log.error('Extend input MeasurementLimit dict: Empty rawParsedMeasurementLimitDict.')
            self.errorStatus = 1
            return

        if len(self.protocolParseInputDict)==0:
            log.error('Extend input MeasurementLimit dict: Empty protocolParseInputDict.')
            self.errorStatus = ErrorStatusEnum.Error
            return

        elif ProtocolCategoryNameEnum.MeasurementLimits.value in self.protocolParseInputDict:
            if self.protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value] is None:
                log.error('Extend input MeasurementLimit dict: protocolParseInputDict["MeasurmentLimit"] is None.')
                self.errorStatus = ErrorStatusEnum.Error
                return
            else:

                if len(self.rawParsedMeasurementLimit.limitsList)!=0:
                    rawParsedMeasLimits = self.rawParsedMeasurementLimit
                    inputMeasLimCats    = self.protocolParseInputDict['MeasurmentLimit']

                    catList = []
                    for limit in rawParsedMeasLimits.limitsList:
                        if limit.catFound==1:
                            catList.append(limit.category)

                    # rawParsedMeasLimitsDict = dict()

                    catSet  = set(catList)
                    catList = list(catSet)

                    auxDict = dict()
                    # rawParsedMeasLimitsDict['MeasurmentLimit']=auxDict

                    for limCat in catList:

                        basisDict = dict()

                        for limit in rawParsedMeasLimits.limitsList:

                            if limit.limitFound==1 and limit.catFound==1 and limit.category==limCat:

                                limitNameArray = limit.name.split('[')
                                limitName = limitNameArray[0]

                                basis = ''

                                if limitName.endswith('rs485') or limitName.endswith('RS485'):
                                    basis = limitName
                                else:
                                    match = re.match(r'(.*\D)?(\d*)$', limitName, re.I)

                                    res = tuple()
                                    if match:
                                        res = match.groups()

                                    basis  = res[0]
                                    suffix = res[1]

                                basisDict[basis] = None

                                auxDict[limCat]  = basisDict

                    outputDict = dict()
                    outputTempDict = inputMeasLimCats
                    outputDict['MeasurmentLimit'] = outputTempDict

                    for rawCatKey, rawCatVal in auxDict.items():

                        outputLimDictPerCat = dict()


                        if rawCatKey not in inputMeasLimCats.keys():
                            outputTempDict[rawCatKey] = rawCatVal

                        else:
                            outputLimDictPerCat = inputMeasLimCats[rawCatKey]
                            inputLimDictPerCat  = copy.deepcopy(inputMeasLimCats[rawCatKey])
                            if len(rawCatVal)!=0:
                                for limit in rawCatVal:


                                    if inputLimDictPerCat is not None:

                                        if isinstance(inputLimDictPerCat, dict):

                                            if limit not in inputLimDictPerCat:


                                                found = 0

                                                for basicName, possibleNames in inputLimDictPerCat.items():

                                                    if limit!=basicName:

                                                        if isinstance(possibleNames,list):

                                                            if limit in possibleNames:
                                                                found = 1
                                                if found==0:
                                                    outputLimDictPerCat[limit] = None


                                            outputTempDict[rawCatKey] = outputLimDictPerCat

                                        else:
                                            log.error('Extend input MeasurementLimit dict :protocolParseInputDict["MeasurmentLimit"], category' + str(rawCatKey) + ', is not of a dict type.')
                                            self.errorStatus = ErrorStatusEnum.Error


                                    else:
                                        outputTempDict[rawCatKey] = auxDict[rawCatKey]
                            else:
                                log.warning('Extend input MeasurementLimit dict : rawParsedMeasurementLimitDict, category' + str(rawCatKey) + ', empty limits list.')
                                self.errorStatus =ErrorStatusEnum.Warning
                    self.extInputMeasurementLimitDict = outputDict
        else:
            log.error('Extend input MeasurementLimit dict: "MeasurmentLimit" key not found at protocolParseInputDict .')
            self.errorStatus = ErrorStatusEnum.Error
            return

    def saveProtocolParseInputDictYAML(self, measLimDict, outputPath):
        log.info('Saving '+str(self.protocolType)+' protocol parse input dict YAML.')

        try:

            with codecs.open(outputPath, 'w+', encoding='iso-8859-1')as yaml_file:

                yaml.dump(measLimDict, yaml_file, default_flow_style=False, sort_keys=False, allow_unicode=True, encoding='iso-8859-1')

        except Exception as exception:
            log.error('Save Protocol Parse Dict YAML, exception: ' + str(exception) + '.')
            self.errorStatus = ErrorStatusEnum.Error

    def loadProtocolParseInputDictYAML(self, filePath):
        try:
            # inputLimitDict=dict()

            log.info('Loading ' +str(self.protocolType)+' protocol prescriptions input file: '+filePath+'.' )
            with open(filePath,'r') as inputFile:
                inputDict = yaml.safe_load(inputFile)

                self.loadProtocolParseInputDict(inputDict)


        except Exception as exception:
            log.error('Load Protocol Parse Input YAML, exception: ' + str(exception) + '.')
            self.errorStatus = ErrorStatusEnum.Error

        pass

    def loadProtocolParseInputDict(self, protocolParseInputDict):

        log.info('Loading ' + str(self.protocolType) + ' protocol prescriptions input dict.')

        if protocolParseInputDict==None:
            protocolParseInputDict = dict()
            protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value] = dict()
            protocolParseInputDict[ProtocolCategoryNameEnum.AbbortCriterions.value]  = dict()
            self.protocolParseInputDict = protocolParseInputDict


        elif isinstance(protocolParseInputDict, dict):
            if ProtocolCategoryNameEnum.MeasurementLimits.value in protocolParseInputDict:

                if not isinstance(protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value], dict):
                    if protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value]==None:

                        protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value] = dict()
                        self.protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimit.value] = protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value]
                        x = 0
                    else:

                        log.error('Load Protocol Parse Input dict, MeasurmentLimit is not of a dict type.')
                        self.errorStatus = ErrorStatusEnum.Error

                else:

                    self.protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value] = protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value]
            else:
                protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value] = dict()
                self.protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value] = protocolParseInputDict[ProtocolCategoryNameEnum.MeasurementLimits.value]

            ####
            if ProtocolCategoryNameEnum.AbbortCriterions.value in protocolParseInputDict:

                if not isinstance(protocolParseInputDict[ProtocolCategoryNameEnum.AbbortCriterions.value], dict):
                    if protocolParseInputDict[ProtocolCategoryNameEnum.AbbortCriterions.value]==None:

                        protocolParseInputDict[ProtocolCategoryNameEnum.AbbortCriterions.value] = dict()
                        self.protocolParseInputDict[ProtocolCategoryNameEnum, AbbortCriterions.value] = protocolParseInputDict[ProtocolCategoryNameEnum.AbbortCriterions.value]
                        x = 0
                    else:

                        log.error('Load Protocol Parse Input dict, AbbortCriterions is not of a dict type.')
                        self.errorStatus = ErrorStatusEnum.Error

                else:

                    self.protocolParseInputDict[ProtocolCategoryNameEnum.AbbortCriterions.value] = protocolParseInputDict[ProtocolCategoryNameEnum.AbbortCriterions.value]
            else:
                protocolParseInputDict[ProtocolCategoryNameEnum.AbbortCriterions.value] = dict()
                self.protocolParseInputDict[ProtocolCategoryNameEnum.AbbortCriterions.value] = protocolParseInputDict[ProtocolCategoryNameEnum.AbbortCriterions.value]


        else:
            log.error('Load Protocol Parse Input dict, no top-level dict availble.')
            self.errorStatus = ErrorStatusEnum.Error

    def generateInputFilteredMeasurementLimit(self,inputLimitDict):

        log.info('Generate '+str(self.protocolType)+' input-filtered MeasurementLimit.')

        if self.errorStatus==1:
            log.error('Generate input-filtered MeasurementLimit: Invalid input.')
            self.errorStatus = ErrorStatusEnum.Error
            return

        else:
            #
            # self.inputFilteredMeasurementLimitDict=dict()
            # tempDict = dict()
            #
            # self.inputFilteredMeasurementLimitDict['MeasurmentLimit'] = tempDict

            ###
            inputMeasurementLimitDict = inputLimitDict[ProtocolCategoryNameEnum.MeasurementLimits.value]

            validLimitObjList = []

            if len(self.rawParsedMeasurementLimit.limitsList)!=0:

                for limitObj in self.rawParsedMeasurementLimit.limitsList:

                    if limitObj.catFound==1 and limitObj.category in inputMeasurementLimitDict:

                        basisLimNamesDict = dict()
                        basisLimNamesDict = inputMeasurementLimitDict[limitObj.category]

                        if basisLimNamesDict is not None:
                            for key, val in basisLimNamesDict.items():

                                if limitObj.limitFound==1:

                                    limitNameArray = limitObj.name.split('[')

                                    limitName = limitNameArray[0]

                                    basis = ''

                                    if limitName.endswith('rs485') or limitName.endswith('RS485'):
                                        basis = limitName
                                    else:
                                        match = re.match(r'(.*\D)?(\d*)$', limitName, re.I)

                                        res = tuple()
                                        if match:
                                            res = match.groups()


                                        basis  = res[0]
                                        suffix = res[1]


                                    if key==basis :
                                        validLimitObjList.append(limitObj)

                                    else:
                                        possibleNamesList = basisLimNamesDict[key]

                                        if possibleNamesList is not None:
                                            if basis in possibleNamesList:
                                                validLimitObjList.append(limitObj)

                                else:
                                    log.warning('Generate input-filtered MeasurementLimit, limit not found in protocol: ' + str(limitObj.name) + ', category: ' + str(limitObj.category) + '.')
                                    self.errorStatus = ErrorStatusEnum.Warning

                    else:
                        log.warning('Generate input-filtered MeasurementLimit, limit not in input prescribtion data: ' + str(limitObj.name) + ', category: ' + str(limitObj.category) + '.')
                        self.errorStatus = ErrorStatusEnum.Warning

                self.inputFilteredMeasurementLimit.limitsList = validLimitObjList

            else:
                log.error('Generate input-filtered MeasurementLimit: Empty rawParsedMeasurementLimit limitsList.')
                self.errorStatus = ErrorStatusEnum.Error
                return

    def rawParseProtocols(self,protocolCategories, protocolPath):

        log.info('Parsing without prescribed input, ' + str(self.protocolType) + ' protocol file: ' + str(protocolPath))

        # try:
        with open(protocolPath,'r') as protocolFile:

            # log.info('Oppening '+str(self.protocolType)+' protocol file: ' +str(protocolPath))

            content = protocolFile.readlines()

            for protocolCategory in protocolCategories:

                if protocolCategory==ProtocolCategoriesEnum.ProtocolHead:

                    protocolHeadStartFound = 0

                    lineNr = 0
                    for line in content:
                        lineNr += 1
                        line = line.strip()

                        if '(1) protocol head:' in line:
                            protocolHeadStartFound = 1

                        if protocolHeadStartFound ==1 and 'Measurment Durations:' not in line :

                            for key,val in self.rawParsedProtocolHead.items():
                                if key.value in line :

                                    self.rawParsedProtocolHead[key].nameFound = 1
                                    self.rawParsedProtocolHead[key].lineNr = lineNr
                                    if ';' not in line:
                                        if ',' in line:
                                            lineList = line.split(',')

                                            for elem in lineList:
                                                if elem!='':
                                                    subList = elem.split(':',1)

                                                    elementValue = ProtocolHeadElementValue(subList[0])
                                                    elementValue.value = subList[1]
                                                    self.rawParsedProtocolHead[key].valuesList.append(elementValue)

                                        else:
                                            lineList = line.split(key.value)

                                            elementValue = ProtocolHeadElementValue(key)
                                            # elementValue.nameFound=1
                                            elementValue.value = lineList[1]
                                            # elementValue.valueFound = 1

                                            self.rawParsedProtocolHead[key].valuesList.append(elementValue)

                                    else:
                                        lineList = line.split(';')
                                        if key.value ==ProtocolHeadEnum.ModularSoftware.value:
                                            for elem in lineList:
                                                if ProtocolHeadEnum.ModularSoftware.value in elem:
                                                    subList = elem.split(':',1)
                                                    elementValue = ProtocolHeadElementValue(subList[0])
                                                    elementValue.value = subList[1]
                                                    self.rawParsedProtocolHead[key].valuesList.append(elementValue)

                                        else:

                                            for index in range(len(lineList)):
                                                if index %2==0:
                                                    if lineList[index]!='':
                                                        elementValue = ProtocolHeadElementValue(lineList[index])
                                                        self.rawParsedProtocolHead[key].valuesList.append(elementValue)
                                                else:
                                                    self.rawParsedProtocolHead[key].valuesList[len(self.rawParsedProtocolHead[key].valuesList)-1].value = lineList[index]

                    for key,val in self.rawParsedProtocolHead.items():

                        if self.rawParsedProtocolHead[key].nameFound==0:
                            self.rawParsedProtocolHead[key].errorStatus = ErrorStatusEnum.Warning

                            log.warning('Protocol Head, element not found: ' + str(self.rawParsedProtocolHead[key].name) + ', line ' + str(self.rawParsedProtocolHead[key].lineNr))

                        else:

                            # decimal check
                            if key==ProtocolHeadEnum.PtuOffsets_Az or key==ProtocolHeadEnum.PtuOffsets_El  or key==ProtocolHeadEnum.MeasDuration or key==ProtocolHeadEnum.MeasDurationV2:

                                for elementValue in self.rawParsedProtocolHead[key].valuesList:

                                    elementValue.nameFound = 1

                                    if sp.checkDecimal(elementValue.value):
                                        elementValue.valueFound = 1
                                        elementValue.value = float(elementValue.value)
                                    else:
                                        elementValue.valueFound = 0
                                        self.rawParsedProtocolHead[key].errorStatus = ErrorStatusEnum.Error

                                        log.warning('Protocol Head, value could not be parsed to float: ' + str(elementValue.name) +', value '+str(elementValue.value) +', line ' + str(self.rawParsedProtocolHead[key].lineNr))
                            elif key==ProtocolHeadEnum.RFSerial:
                                i = 0
                                for elementValue in self.rawParsedProtocolHead[key].valuesList:
                                    elementValue.nameFound = 1

                                    if i==0:
                                        elementValue.valueFound = 1

                                    elif i==1:
                                        if sp.checkDecimal(elementValue.value):
                                            elementValue.valueFound = 1
                                            elementValue.value = float(elementValue.value)
                                        else:
                                            elementValue.valueFound = 0
                                            self.rawParsedProtocolHead[key].errorStatus = ErrorStatusEnum.Error

                                            log.warning('Protocol Head, value could not be parsed to float: ' + str(elementValue.name) +', value '+str(elementValue.value) +', line ' + str(self.rawParsedProtocolHead[key].lineNr))

                            #string check
                            elif key==ProtocolHeadEnum.CalibrationDspSoftwareInfo or key==ProtocolHeadEnum.CalibrationDspSoftwareCompileInfo or key==ProtocolHeadEnum.CalStart or key==ProtocolHeadEnum.CalEnd:
                                for elementValue in self.rawParsedProtocolHead[key].valuesList:
                                    elementValue.nameFound  = 1
                                    elementValue.valueFound = 1

                            elif key==ProtocolHeadEnum.ModularSoftware:
                                for elementValue in self.rawParsedProtocolHead[key].valuesList:
                                    elementValue.nameFound = 1
                                    if elementValue.value!='':
                                        elementValue.valueFound = 1

                            elif key==ProtocolHeadEnum.BurnInAzi or key==ProtocolHeadEnum.BurnInEle or key==ProtocolHeadEnum.BurnInSpeed:
                                for elementValue in self.rawParsedProtocolHead[key].valuesList:
                                    elementValue.valueFound = 1
                                    elementValue.value = elementValue.value

                if protocolCategory==ProtocolCategoriesEnum.MeasurementDurations:

                    measDurationsStartFound  = 0
                    measDurationsStart1Found = 0
                    lineNr = 0
                    for line in content:
                        lineNr += 1
                        line = line.strip()
                        line = line.replace('\n','')

                        if 'Measurment Durations:' in line:
                            measDurationsStartFound = 1

                        if measDurationsStartFound and '-------' in line:
                            measDurationsStart1Found = 1

                        if measDurationsStartFound==1 and measDurationsStart1Found==1 and '-------' not in line :
                            if line!='':
                                line = line.replace(';','')
                                line = line.strip()
                                lineList = line.split(':',1)
                                measurementDuration = MeasurementDuration()


                                if lineList[0]!='':
                                    measurementDuration.name = lineList[0]
                                    measurementDuration.nameFound = 1
                                else:
                                    measurementDuration.errorStatus = ErrorStatusEnum.Warning
                                    log.warning('Measurement Duration:, Name not found: ' + str(lineList[0]) + ', line ' + str(lineNr))

                                subString = lineList[1]
                                subList   = subString.split('Start:')
                                value     = subList[0].strip()
                                if sp.checkDecimal(value):
                                    measurementDuration.value = float(value)
                                    measurementDuration.valueFound = 1

                                else:
                                    measurementDuration.valueFound  = 0
                                    measurementDuration.errorStatus = ErrorStatusEnum.Warning
                                    log.warning('Measurement Duration:, Value could not be parsed to float: ' + str(value) + ', line ' + str(lineNr))

                                subList1 = subList[1].split('End:')
                                start  = subList1[0].strip()
                                if subList1[0]!='':
                                    measurementDuration.start = start
                                    measurementDuration.startFound = 1

                                else:
                                    measurementDuration.startFound = 0
                                    measurementDuration.errorStatus = ErrorStatusEnum.Warning
                                    log.warning('Measurement Duration, Start not found: ' + str(start) + ', line ' + str(lineNr))

                                end = subList1[1].strip()
                                if end!='':
                                    measurementDuration.end = end
                                    measurementDuration.endFound = 1

                                else:
                                    measurementDuration.endFound = 0
                                    measurementDuration.errorStatus = ErrorStatusEnum.Warning
                                    log.warning('Measurement Duration, End not found: ' + str(end) + ', line ' + str(lineNr))


                                self.rawParsedMeasurementDurations.append(measurementDuration)
                            else:
                                break
                                x=0

                if protocolCategory==ProtocolCategoriesEnum.MeasSettings:

                    measSettingsStartFound  = 0
                    measSettingsStart1Found = 0
                    measSettingsStart2Found = 0
                    lineNr = 0
                    for line in content:
                        lineNr += 1
                        line = line.strip()
                        line = line.replace('\n', '')

                        if 'Meas Settings' in line:
                            measSettingsStartFound = 1

                        if measSettingsStartFound and '-------' in line:
                            measSettingsStart1Found = 1

                        if measSettingsStartFound and measSettingsStart1Found and line=='':
                            measSettingsStart2Found = 1

                        if measSettingsStartFound==1 and measSettingsStart1Found==1 and measSettingsStart2Found==1 and line!='':
                            if '-----' not in line:
                                line = line.strip()
                                lineList = line.split(';')
                                measSetting = MeasSetting()

                                name = lineList[0].strip()
                                if name !='':
                                    measSetting.name = name
                                    measSetting.nameFound =1
                                else:
                                    measurementDuration.errorStatus = ErrorStatusEnum.Warning
                                    log.warning('Measurement Settings, Name not found: ' + str(name) + ', line ' + str(lineNr))

                                value = lineList[1].strip()
                                if value!='':
                                    measSetting.value = value
                                    measSetting.valueFound = 1

                                else:
                                    measSetting.valueFound = 0
                                    measSetting.errorStatus = ErrorStatusEnum.Warning
                                    log.warning('Measurement Settings, Value not found: ' + str(value) + ', line ' + str(lineNr))

                                comment=lineList[2]
                                if comment!='':
                                    measSetting.comment = comment
                                    measSetting.commentFound = 1
                                else:
                                    measSetting.errorStatus = ErrorStatusEnum.Warning
                                    log.warning('Measurement Settings, Comment not found: ' + str(comment) + ', line ' + str(lineNr))

                                self.rawParsedMeasSettings.append(measSetting)
                            else:
                                break


                if  protocolCategory== ProtocolCategoriesEnum.MeasurementLimits:

                        # content=bytearray(protocolFile.read())
                        # charDet = chardet.detect(content)
                        # encoing = charDet['encoding']

                        measLimitsStartFound = 0

                        lineNr=0
                        measurementLimit=MeasurementLimit()
                        for line in content:
                            lineNr+=1
                            line = line.strip()
                            lineList = line.split(';')

                            # # New
                            # if 'Burn16a_BurnInMeasSpeedList[km/h]' in line:
                            #     lineListBurn = lineList[1].replace(' ', '')
                            #     lineListBurn = lineListBurn.split('|')
                            #     for elemIdx in range(len(lineListBurn)):
                            #         BurnIn= BurnInMeas()
                            #         if sp.checkDecimal(lineListBurn[elemIdx]):
                            #             lineList[0] = lineList[0].replace(' ', '')
                            #             BurnIn.name  = str(lineList[0])+str(elemIdx)
                            #             BurnIn.value = lineListBurn[elemIdx]
                            #         measurementLimit.limitsList.append(BurnIn)

                            if '(9) limit values results:' in line:
                                measLimitsStartFound=1

                            elif measLimitsStartFound==1 and len(lineList)!=1 and lineList[0]!='':

                                if len(lineList)<7:
                                    log.warning('Not sufficient elements found for MeasurmentLimit, line '+ str(lineNr))
                                    self.errorStatus=ErrorStatusEnum.Error
                                    break

                                limit = Limit()
                                for elemIdx in range(len(lineList)):
                                    if 'MeasurmentLimit' in lineList[elemIdx]:
                                        measLimitCatList=lineList[elemIdx].split(':')
                                        cat=measLimitCatList[1].strip(' ')
                                        limit.category=cat
                                        limit.catFound=1
                                    elif limit.catFound==1 and limit.limitFound!=1:
                                        limit.name=lineList[elemIdx].strip('=')
                                        limit.limitFound=1
                                    elif limit.catFound==1 and limit.limitFound==1:
                                        if limit.minFound!=1 and limit.maxFound!=1 and limit.valueFound!=1:
                                            lineList[elemIdx].replace(',','.')
                                            if sp.checkDecimal(lineList[elemIdx]):
                                                limit.value=float(lineList[elemIdx])
                                                limit.valueFound=1
                                            else:
                                                log.warning('Value could not be parsed to float: '+str(lineList[elemIdx])+', line '+str(lineNr))
                                                limit.errorStatus=ErrorStatusEnum.Error
                                                limit.value = str(lineList[elemIdx])
                                                break
                                        elif limit.maxFound!=1 and limit.minFound!=1 and limit.valueFound==1:
                                            lineList[elemIdx].replace(',','.')
                                            maxList = lineList[elemIdx].split(':')
                                            maxList[1] = maxList[1].replace(',','.').replace(' ', '')
                                            max = maxList[1]
                                            if sp.checkDecimal(max):

                                                limit.max = float(maxList[1])
                                                limit.maxFound = 1
                                            else:
                                                log.error('Max could not be parsed to float: ' + str(maxList[1]) + ', line ' + str(lineNr))
                                                limit.errorStatus = ErrorStatusEnum.Error
                                                limit.max = str(maxList[1])
                                                break

                                        elif limit.maxFound==1 and limit.minFound!=1 and limit.valueFound==1:
                                            
                                            minList = lineList[elemIdx].split(':')
                                            minList[1] = minList[1].replace(',','.').replace(' ', '')
                                            min = minList[1]
                                            if sp.checkDecimal(min):

                                                limit.min = float(minList[1])
                                                limit.minFound = 1
                                            else:
                                                log.warning('Min could not be parsed to float: ' + str(
                                                    minList[1]) + ', line ' + str(lineNr))
                                                limit.errorStatus = ErrorStatusEnum.Error
                                                limit.min = str(minList[1])
                                                break

                                        # elif limit.minFound==1 and limit.maxFound==1 and limit.valueFound==1 and 'MeasResult' in lineList[elemIdx]:
                                        elif limit.minFound==1 and limit.maxFound==1 and limit.valueFound==1 and 'MeasResultMeasOk' in lineList[elemIdx]:
                                            limit.measResultStringFound = 1
                                        elif limit.minFound==1 and limit.maxFound==1 and limit.valueFound==1 and limit.measResultStringFound==1 and limit.measResultFound!=1 and limit.commentFound!=1:
                                            lineList[elemIdx].replace(',','.')
                                            if sp.checkInt(lineList[elemIdx]):
                                                measResult=int(lineList[elemIdx])
                                                limit.measResult=MeasResultEnum(measResult)
                                                limit.measResultFound=1
                                            else:
                                                log.warning('MeasResult could not be parsed to int: ' + str(lineList[elemIdx]) + ', line ' + str(lineNr))
                                                limit.errorStatus = ErrorStatusEnum.Error
                                                limit.measResult=str(lineList[elemIdx])
                                                break
                                        elif limit.minFound==1 and limit.maxFound==1 and limit.valueFound==1 and limit.measResultStringFound==1 and limit.measResultFound==1 and limit.commentFound!=1:
                                            if lineList[elemIdx]!='':
                                                limit.commentFound=1
                                                limit.comment=lineList[elemIdx]
                                        elif  limit.minFound==1 and limit.maxFound==1 and limit.valueFound==1 and limit.measResultStringFound==1 and limit.measResultFound==1 and limit.commentFound==1:
                                            if lineList[elemIdx]!='':
                                                limit.errorStatus=ErrorStatusEnum.Warning
                                                log.warning('Extra argument found: ' + str(lineList[elemIdx]) + ', line ' + str(lineNr))
                                            break

                                measurementLimit.limitsList.append(limit)
                                self.rawParsedMeasurementLimit=measurementLimit

                            elif measLimitsStartFound==1 and len(lineList)==1 and lineList[0]=='':
                                break

                elif protocolCategory==ProtocolCategoriesEnum.Measurements:


                    bassicParamNamesFound = 0
                    basicParamNamesSearch = 0

                    for measTypeKey, measTypeVal in self.rawParsedMeasurements.items():

                        if measTypeVal.type==MeasurementEnum.TxAntPowerCorrCurve:
                            basicParamNamesSearch = BasicMeasurementParamSearchEnum.AfterOtherParams
                        elif measTypeVal.type==MeasurementEnum.TxAntRawFrequencyTable_Measpoints or measTypeKey==MeasurementEnum.TxAntCtapRawFreqSelectSearch:
                            basicParamNamesSearch = BasicMeasurementParamSearchEnum.RightAfterMeasTypeTitle

                        lineNr = 0
                        for line in content:
                            lineNr += 1
                            line = line.strip()

                            if measTypeVal.name==line or line in measTypeVal.possibleNames:
                                measTypeVal.actualName=line
                                measTypeVal.nameFound=1
                            elif measTypeVal.nameFound==1 and basicParamNamesSearch ==BasicMeasurementParamSearchEnum.AfterOtherParams:

                                if measTypeVal.type ==MeasurementEnum.TxAntPowerCorrCurve:

                                    lineList = line.split(';')

                                    if lineList[0]in TxAntPowerCorrCurvePLVU_PossibleNames:
                                        measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelValuesUsed].nameFound = 1
                                        if sp.checkInt(lineList[1]):
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelValuesUsed].valuesFound=1
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelValuesUsed].values.append(int(lineList[1]))
                                        else:
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelValuesUsed].errorStatus=ErrorStatusEnum.Error
                                            log.warning('Measurements, parsing to int failed, measurements type: ' + measTypeKey + ', parameter: ' + TxAntPowerCorrCurveEnum.PowermeterLevelValuesUsed.value + ', line ' + str(lineNr))

                                    elif lineList[0] in TxAntPowerCorrCurvePLRA_PossibleNames:
                                        measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelRecalcActive].nameFound =1
                                        if sp.checkInt(lineList[1]):
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelRecalcActive].valuesFound = 1
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelRecalcActive].values.append(int(lineList[1]))
                                        else:
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelRecalcActive].errorStatus = ErrorStatusEnum.Error
                                            log.warning('Measurements, parsing to int failed, measurements type: ' + measTypeKey + ', parameter: ' + TxAntPowerCorrCurveEnum.PowermeterLevelRecalcActive.value + ', line ' + str(lineNr))

                                    elif lineList[0] in  TxAntPowerCorrCurvePLCM_PossibleNames:
                                        measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod].nameFound=1
                                        if sp.checkInt(lineList[1]):
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod].valuesFound = 1
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod].values.append(int(lineList[1]))
                                        else:
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod].errorStatus = ErrorStatusEnum.Error
                                            log.warning('Measurements, parsing to int failed, measurements type: ' + measTypeKey + ', parameter: ' + TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod.value + ', line ' + str(lineNr))

                                        if lineList[2]!='':
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod].values.append(lineList[2])
                                        else:
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod].valuesFound = 0
                                            measTypeVal.parameterMeasurementValueDict[TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod].errorStatus = ErrorStatusEnum.Error
                                            log.warning('Measurements, measurements type: ' + measTypeKey + ', parameter value not found: ' + TxAntPowerCorrCurveEnum.PowermeterLevelCorrMethod.value + ', line ' + str(lineNr))

                                        if '3eTx' in measTypeVal.actualName:
                                            measTypeVal.searchBasicParamNamesWithoutHeader(lineNr)

                                    elif measTypeVal.bassicParamNamesFound==0:
                                        # if measTypeVal.searchPatternInPossibleNames('3eTx', measTypeVal.possibleNames):
                                        #     measTypeVal.searchBasicParamNamesWithoutHeader(lineNr)
                                        # else:
                                        measTypeVal.searchBasicParamNames(line,lineNr)

                                    elif measTypeVal.bassicParamNamesFound==1:

                                        lineList = line.split(';')

                                        if len(lineList)==1 and lineList[0]=='':
                                            break
                                        else:
                                           measTypeVal.parseBasicParamNames(lineList, lineNr)


                            elif measTypeVal.nameFound==1 and basicParamNamesSearch==BasicMeasurementParamSearchEnum.RightAfterMeasTypeTitle:
                                if '3aTx' in measTypeVal.actualName and measTypeVal.bassicParamNamesFound==0:
                                    measTypeVal.searchBasicParamNamesWithoutHeader(lineNr)
                                    # measTypeVal.noHeaderProcesed =1
                                if measTypeVal.bassicParamNamesFound==0:
                                    # if measTypeVal.searchPatternInPossibleNames('3aTx',measTypeVal.possibleNames):
                                    #     measTypeVal.searchBasicParamNamesWithoutHeader(lineNr)
                                    # else:
                                    measTypeVal.searchBasicParamNames(line, lineNr)

                                elif measTypeVal.bassicParamNamesFound==1:

                                    lineList = line.split(';')

                                    if len(lineList)==1 and lineList[0]=='':
                                        break
                                    else:
                                        measTypeVal.parseBasicParamNames(lineList, lineNr)

                            elif measTypeVal.nameFound==1 and basicParamNamesSearch==BasicMeasurementParamSearchEnum.NotAtAll:
                                break

                    for measTypeKey,measTypeObj in self.rawParsedMeasurements.items():

                        if measTypeObj.nameFound !=1:

                            measTypeObj.errorStatus=ErrorStatusEnum.Warning
                            log.warning('Measurements, name not found, measurements type: ' + measTypeKey + '.')

                        for paramKey, paramObj in measTypeObj.parameterMeasurementValueDict.items():

                            if paramObj.errorStatus==ErrorStatusEnum.Error:
                                measTypeObj.errorStatus = ErrorStatusEnum.Error

                elif protocolCategory ==ProtocolCategoriesEnum.AbbortCriterions:
                    abbortCriterionsStartFound = 0
                    abbortCriterionsString='(10) abort criterions - results:'

                    lineNr = 0
                    abbortCriterions = AbbortCriterions()
                    insufValsHeader = 0

                    for line in content:
                        lineNr += 1
                        line = line.strip()
                        lineList = line.split(';')

                        abbortCriterion = AbbortCriterion()

                        if abbortCriterionsString in line:
                            abbortCriterionsStartFound = 1

                        elif abbortCriterionsStartFound==1 and len(lineList)!=1 and lineList[0]!='':

                            headerLine=1
                            if lineList[0]=='measured values':
                                headerLine =0

                            if headerLine==1:
                                if len(lineList) < 4:
                                    log.warning('Not sufficient elements found for Abbort Criterion header, line '+ str(lineNr))
                                    self.errorStatus=ErrorStatusEnum.Warning
                                    insufValsHeader=1
                                    break
                                else:
                                    abbortCriterion = AbbortCriterion()
                                    if sp.checkInt(lineList[0]):
                                        abbortCriterion.id = int(lineList[0])
                                        abbortCriterion.numberFound=1
                                    else:
                                        log.warning('Abbort Criterion number could not be parsed to int: ' + str(lineList[0]) + ', line ' + str(lineNr))
                                        abbortCriterion.errorStatus=ErrorStatusEnum.Error
                                    if lineList[1]!='':
                                        abbortCriterion.name=lineList[1].strip()
                                        abbortCriterion.nameFound=1
                                    else:
                                        log.warning('Abbort Criterion name not found, line: '+ str(lineNr))
                                        abbortCriterion.errorStatus=ErrorStatusEnum.Error
                                    if lineList[2]!='':
                                        abbortCriterion.statusCode=lineList[2]
                                        abbortCriterion.statusCodeFound=1
                                    else:
                                        log.warning('Abbort Criterion status code not found, line: '+ str(lineNr))
                                        abbortCriterion.errorStatus=ErrorStatusEnum.Error

                                    insufValsHeader = 0
                                    abbortCriterions.abbortCriterionList.append(abbortCriterion)

                            else:
                                if insufValsHeader ==1:
                                    break
                                else:
                                    if len(lineList)<2:
                                        log.warning('Not sufficient elements found for Abbort Criterion measured values, line ' + str(lineNr))
                                        self.errorStatus = ErrorStatusEnum.Error
                                        break
                                    else:
                                        valueList=[]
                                        parseError=0
                                        for valueIdx in range(1, len(lineList)):
                                            if valueIdx !=len(lineList)-1:
                                                if sp.checkDecimal(lineList[valueIdx]):
                                                    value = float(lineList[valueIdx])
                                                    valueList.append(value)
                                                else:
                                                    parseError=1
                                            else:
                                                if lineList[valueIdx]!='':
                                                    self.errorStatus=ErrorStatusEnum.Error
                                                    log.warning('Abbort Criterion values could not be parsed, unexpected character at end: ' + str(lineList[0]) + ', line ' + str(lineNr))

                                        if parseError==1:
                                            log.warning('Abbort Criterion values could not be parsed to float: ' + str(lineList[0]) + ', line ' + str(lineNr))
                                            self.errorStatus = ErrorStatusEnum.Error
                                        else:
                                            abbortCriterions.abbortCriterionList[len(abbortCriterions.abbortCriterionList)-1].valuesFound=1
                                            abbortCriterions.abbortCriterionList[len(abbortCriterions.abbortCriterionList)-1].values=valueList

                    self.rawParsedAbbortCriterions = abbortCriterions

        # except Exception as exception:
        #     log.error('Opening protocol file error: '+str(exception)+'.')
        #     self.errorStatus = ErrorStatusEnum.Error

class ParseInputLimitsGroups:
    def __init__(self):
        self.burnInErrorStatus = 0
        self.calErrorStatus = 0

        self.burnInInputLimitsGroupsDict=dict()
        self.calInputLimitsGroupsDict=dict()

        self.burnInExtInputLimitsGroupsDict = dict()
        self.calExtInputLimitsGroupsDict = dict()

    def loadBurnInInputLimitsGroupsDictYAML(self, filePath):

        self.burnInInputLimitsGroupsDict,self.burnInErrorStatus= self.loadInputLimitsGroupsDictYAML(filePath)

    def loadCalInputLimitsGroupsDictYAML(self, filePath):

        self.calInputLimitsGroupsDict,self.calErrorStatus= self.loadInputLimitsGroupsDictYAML(filePath)

    def loadInputLimitsGroupsDictYAML(self, filePath):
        errorStatus=0
        # try:
            # inputLimitDict=dict()
        with open(filePath,'r') as inputFile:
            inputLimitDict=yaml.safe_load(inputFile)

            if  inputLimitDict==None:
                inputLimitDict=dict()

                # inputLimitDict['BurnIn']=[]
                # inputLimitDict['Calibration'] = []
                # self.inputLimitsGroupsDict = inputLimitDict


            elif isinstance(inputLimitDict,dict):
                for key,val in inputLimitDict.items():
                    if not isinstance(val,dict):
                        log.error('Load Input Limits YAML, Group '+str(val)+' does not contain a nested dict.')
                        errorStatus = 1
                    else:
                        if 'Limits' in val:
                            if not isinstance(val['Limits'], dict):
                                log.error('Load Input Limits YAML, Group ' + str(val) + ', "Limit" key is not of a dict type.')
                                errorStatus = 1
                            else:
                                limitsDict = val['Limits']
                                if 'ExactMatch' not in limitsDict:
                                    limitsDict['ExactMatch']=[]
                                else:
                                    if not isinstance(limitsDict['ExactMatch'],list):
                                        log.error('Load Input Limits YAML, Group ' + str(val) + ', "ExactMatch" is not of a list type.')
                                        errorStatus = 1
                                    else:
                                        inputLimitDict[key]['Limits']['ExactMatch']= limitsDict['ExactMatch']

                                if 'Contains' not in limitsDict:
                                    limitsDict['Contains']=[]
                                else:
                                    if not isinstance(limitsDict['Contains'],list):
                                        log.error('Load Input Limits YAML, Group ' + str(val) + ', "Contains" is not of a list type.')
                                        errorStatus = 1
                                    else:
                                        inputLimitDict[key]['Limits']['Contains']= limitsDict['Contains']
                        else:
                            inputLimitDict[key]['Limits']=dict()
                            inputLimitDict[key]['Limits']['ExactMatch']=[]
                            inputLimitDict[key]['Limits']['Contains'] = []


                ##### ##### #####
                #     if 'BurnIn' in inputLimitDict:
                #
                #         if not isinstance(inputLimitDict['BurnIn'],list):
                #             if inputLimitDict['BurnIn']==None:
                #
                #                 inputLimitDict['BurnIn']=[]
                #
                #             else:
                #
                #                 log.error('Load Input Limits YAML, BurnIn Limits Group is not of a list type.')
                #                 errorStatus=1
                #
                #         else:
                #             self.inputLimitsGroupsDict = inputLimitDict['BurnIn']
                #     else:
                #         inputLimitDict['BurnIn']=[]
                #
                #     if 'Calibration' in inputLimitDict:
                #
                #         if not isinstance(inputLimitDict['Calibration'], list):
                #             if inputLimitDict['Calibration']==None:
                #
                #                 inputLimitDict['Calibration'] = []
                #
                #             else:
                #
                #                 log.error('Load Input Limits YAML, Caliibration Limits Group is not of a list type.')
                #                 errorStatus = 1
                #
                #         else:
                #             self.inputLimitsGroupsDict = inputLimitDict['Calibration']
                #     else:
                #         inputLimitDict['Calibration'] = []
                #
                #     if errorStatus !=1:
                #         self.inputLimitsGroupsDict = inputLimitDict
                #
                # else:
                #     log.error('Load Input Limits YAML, no top-level dict availble.')
                #     errorStatus = 1

        # except Exception as exception:
        #     log.error('Load Input Limits YAML, exception: ' + str(exception) + '.')
        #     errorStatus=1

        return inputLimitDict, errorStatus

    def extendBurnInInputLimitsGroupsDict(self, groupNamesList):
        protType='BurnIn'

        self.burnInExtInputLimitsGroupsDict, self.burnInErrorStatus =self.extendInputLimitsGroupsDict(groupNamesList, self.burnInInputLimitsGroupsDict, protType)

    def extendCalInputLimitsGroupsDict(self, groupNamesList):
        protType='Calibration'

        self.calExtInputLimitsGroupsDict, self.calErrorStatus =self.extendInputLimitsGroupsDict(groupNamesList, self.calInputLimitsGroupsDict, protType)

    def extendInputLimitsGroupsDict(self, groupNamesDictList, inputLimitsGroupsDict, protType):

        errorStatus=0
        protTypeStr='BurnIn'
        if protType !='BurnIn':
            protTypeStr='Calibration'

        if len(groupNamesDictList)==0:
            log.error('Extend input Limits Groups dict: Empty list of '+protTypeStr +' limits group names dicts.')
            errorStatus = 1

        # if len(inputLimitsGroupsDict)==0:
        #     log.error('Extend input Limits Groups dict: Empty '+protTypeStr+' extended input limits groups dict.')
        #     errorStatus=1

        if errorStatus!=1:

            outputDict=dict()
            for key, val in inputLimitsGroupsDict.items():
                outputDict[key] =inputLimitsGroupsDict[key]

            for groupNameDict in groupNamesDictList:

                groupName = groupNameDict['groupName']

                if groupName not in inputLimitsGroupsDict:
                    outputDict[groupName]=dict()

                    auxLimDict=groupNameDict['Limits']

                    outputDict[groupName]['Limits']=auxLimDict

                # else:
                #     outputDict[groupName] =inputLimitsGroupsDict[groupName]


                # outputDict['BurnIn']=inputLimitsGroupsDict['BurnIn']
                #
                # for burnInBasis in burnInBasisList:
                #     if burnInBasis not in self.inputLimitsGroupsDict['BurnIn']:
                #         outputDict['BurnIn'].append(burnInBasis)
                #
                # for calBasis in calBasisList:
                #     if calBasis not in self.inputLimitsGroupsDict['Calibration']:
                #         outputDict['Calibration'].append(calBasis)
                #
            return outputDict, errorStatus

    def saveToActualBurnInLimitsGroupsDict(self, limitsGroup, alreadyExtended):

        limNameDict= self.saveToActualLimitsGroupsDict(limitsGroup)

        if alreadyExtended ==0:
            self.burnInInputLimitsGroupsDict[limitsGroup.name] =limNameDict
        else:
            self.burnInExtInputLimitsGroupsDict[limitsGroup.name] =limNameDict

    def saveToActualCalLimitsGroupsDict(self, limitsGroup, alreadyExtended):

        limNameDict= self.saveToActualLimitsGroupsDict(limitsGroup)

        if alreadyExtended ==0:
            self.calInputLimitsGroupsDict[limitsGroup.name] =limNameDict
        else:
            self.calExtInputLimitsGroupsDict[limitsGroup.name] =limNameDict

    def saveToActualLimitsGroupsDict(self,limitsGroup):

        limNameDict=dict()
        limitsDict = dict()

        containsList = []
        exactMatchList=[]
        for limitRowMatchDict in limitsGroup.limitsRowMatchDictList:

            limitRow=limitRowMatchDict['row']

            if limitRowMatchDict['match']=='ExactMatch':
                exactMatchList.append(limitRow.limitName)
            else:
                containsList.append(limitRow.limitName)


        limitsDict['Contains']=containsList
        limitsDict['ExactMatch']=exactMatchList

        limNameDict['Limits']=limitsDict

        return limNameDict

    def saveLimitsGroupsDictYAML(self, limGroupsDict, outputPath):
        with codecs.open(outputPath, 'w+', encoding='iso-8859-1')as yaml_file:

            yaml.dump(limGroupsDict, yaml_file, default_flow_style=False, sort_keys=False, allow_unicode=True, encoding='iso-8859-1')

def generateInputLimitsFromFile(filePath,limExportPath):
    try:
        with open(filePath, 'r') as protocolFile:

            content = protocolFile.read().replace('\n', '')


            contentList = content.split(';MeasurmentLimit: ')
            del contentList[0]

            pairDictList=[]
            catList=[]
            limList=[]

            for entry in contentList:
                pairDict = dict()

                entryList = entry.split(';')

                category=entryList[0]
                limit =entryList[1]

                limitL = limit.split('[')

                limit = limitL[0]

                limit = limit.replace('=', '')

                basis = ''

                if limit.endswith('rs485') or limit.endswith('RS485'):
                    basis = limit
                else:
                    match = re.match(r'(.*\D)?(\d*)$', limit, re.I)

                    res = tuple()
                    if match:
                        res = match.groups()

                    basis = res[0]
                    suffix = res[1]

                pairDict['limit']=basis
                pairDict['category']=category

                catList.append(category)
                pairDictList.append(pairDict)

                a=0

            catList=set(catList)
            catList=list(catList)

            catDict = dict()
            for category in catList:

                limDict = dict()

                for pairDict in pairDictList:
                    cat = pairDict['category']
                    lim = pairDict['limit']

                    if cat==category and lim not in limDict.keys():

                        limDict[lim] = None

                catDict[category] = limDict


            measLimitDict = dict()
            measLimitDict['MeasurmentLimit'] = catDict


        with codecs.open(limExportPath, 'w+', encoding='iso-8859-1')as yaml_file:

            yaml.dump(measLimitDict, yaml_file, default_flow_style=False, sort_keys=False, allow_unicode=True, encoding='iso-8859-1')

    except Exception as exception:
        log.error('Opening protocol file: ' + str(exception) + '.')


# o=ParseProtocols()

# # o.loadInputLimitsDictYAML('C:\Projects\StatisticToolPythonQtGUI\CalLimits.yaml')
# o.loadInputLimitsDictYAML('C:\Projects\StatisticToolPythonQtGUI\CalLimits.yaml')
#
# o.loadInputLimitsDictYAML('C:\Arbeit\BurnInLimits.yaml')

# catList=[ProtocolCategoriesEnum.ProtocolHead, ProtocolCategoriesEnum.MeasurementLimits,ProtocolCategoriesEnum.Measurements, ProtocolCategoriesEnum.AbbortCriterions]
#
# o.rawParseProtocols(catList,r'C:\Projects\StatisticToolPythonQtGUI\output_certificates\0x0002EA1A\Calibration\2019-03-12-10-23-27_Tuner0_rf0x0002E125_dsp0x0002AFC9_Protocol.txt')

# o.rawParsedProtocolHead[ProtocolHead.CalEnd]='2'
# o.rawParsedProtocolHead['CalStart:']='2'
# o.generateInputFilteredMeasurementLimit(o.inputLimitDict)
# o.extendInputMeasurementLimitDict()
# o.generateInputFilteredMeasurementLimit(o.extInputMeasurementLimitDict)
# o.inputFilteredMeasurementLimitDict=o.generateMeasurementLimitDict(o.inputFilteredMeasurementLimit,o.inputLimitDict)
#
#
#
# # o.inputFilteredMeasurementLimitDict=o.generateMeasurementLimitDict(o.inputFilteredMeasurementLimit)
#
# o.saveMeasurementLimitsDictYAML(o.extInputMeasurementLimitDict, 'C:\Arbeit\limoutput.yaml')
#
#
# # o.saveMeasurementLimitsDictYAML(o.extInputMeasurementLimitDict, 'C:\Arbeit\extInputLimits.yaml')
# a=0

# generateInputLimitsFromFile('C:\Arbeit\CalWriteLimitsTest.txt','C:\Projects\StatisticToolPythonQtGUI\CalLimits.yaml')



import yaml
import os

import matplotlib.pyplot as plt
import numpy as np
import logconfig
import auxiliary as aux
import loaddata as ld
import antpat_v5_data_processing as dp
import copy
import yaml


bUseGui=True

########## Configuration ###########
#absPath=r'C:\Users\Raza\Desktop\StatisticToolPython_QtGUI\PT00012245_20349KAC0KC00255_135233\2020-03-21-13-52-48_Tuner0_PT00012245_MANUAL_SN_Protocol.txt'
absPath=r'C:\Arbeit\Mobis_beispiel'
# absPath=r'\\fs9\MDSMS0\Kalibrierung\Kalibrierung_USM\Archiv\2015\2015_Typ31_STM\intern_SMS_56xT31ErstmusterZollner\UMRR-0A.49.03-00024748'
absPath=r'C:\Arbeit\noTxAnt\noDiag'
absPath = r'C:\Arbeit\noDiag_mobis'
absPath = r'C:\Arbeit\noDiag_mobis\UMRR-0A.49.03-00024748'
# absPath = r'C:\OpenAccess\von_Qi\CalData_T150ES3'
# absPath=r'C:\OpenAccess\von_Axel\Antenna_Patterns\test'
# absPath=r'C:\Arbeit\intern_8xUMRR-0A4903-1D0907fürRadarVision'
# absPath = r'\\fs9\MDSMS0\Kalibrierung\Kalibrierung_USM\Results\Typ_0x2AXAXX\2018\intern_350xUMRR-0C0304-2A2A02Nov2018'
# absPath =r'C:\Arbeit\two_antenna_types_test\rfNA_dsp0x00000007'
absPath = r'C:\Arbeit\qi_axel'
absPath = r'C:\Arbeit\ALPSsample'
absPath = r'\\ad.smartmicro.de\Data\Production\Kalibrierung\Results\Typ_0x84XXXX_UMRR-11\2019\intern_90xUMRR-110004-840101ohneLogoJuli2019'
absPath =r'\\ad.smartmicro.de\Data\Messdaten\MDSMS0\Kalibrierung\Test_TDAX_Kalibrierung\2019-11-27DevelopUmrr12T48_allWF\Messdaten'
absPath=r'C:\Hafiz\EOL_1\data'
# Script setup
setup ={
    "auto": 0, # Set 0 for manual configuration. Set 1 for automatic generation of antenna pattern tables.
    "useSensorMuster": 0,  # Set 1 to search for sensors named by specific muster, otherwise 0.for
    "sensorMusterList": []  # Sensor Muster parts list
}

# Configuration for the files WITH diaram type info.

# Chirp configuration
chirpConfig = {
    "chirp0": 1,
    "chirp1": 0,
    "chirp2": 0,
    "chirp3": 0,
}
## Header ##
antPatHeader = {

    # Name
    "type": "antennaPattern",

    # # Port identifier
    # "identifier": 36,

    # # Version
    # "majorVersion": 2,
    # "minorVersion": 0,

    # # Date
    # "year": 2020,
    # "month": 1,
    # "day": 28,

    # # Antenna information
    # "rfeGeneration": 44,
    # "rfeModification": 56,
    # "rfeRevision": 0,

}


## Data Tables ##
# "patternType": 1,     //# Antenna pattern type (0: invalid; 1: TX - RX; 2: TX; 3: RX)
# "rotationAxis": 2,    //# Antenna rotation axis (0: invalid; 1: z, azimuth; 2: y, elevation; 3: x, polarization)
# "nofApplicableTxAnt": 4,      //# [1] Number of TX antennas the pattern is applicable to
# "nofApplicableRxAnt": 4,      //# [1] Number of RX antennas the pattern is applicable to
# "applTxAntInd": [0, 0, 0, 0],     //# Indices of the TX antennas the pattern is applicable to
# "applRxAntInd": [0, 1, 2, 3],     //# Indices of the RX antennas the pattern is applicable to
# "gainCalcMethod": 1       //# [1] Method of data calculation (0: invalid, 1: step-wise, 2: interpolation )

##Port Data##
portData ={

    # Number of tables to generate
    "nofDataTables": 1,
    "version": 1
}

antPatTable0 = {
    "antenna_pattern_idx": 0,
    "patternType": 1,
    "rotationAxis": 1,
    "nofApplicableTxAnt": 1,
    "nofApplicableRxAnt": 4,
    "applTxAntInd": [0],
    "applRxAntInd": [0,1,2,3],
    "gainCalcMethod": 1
}

antPatTable1 = {
    "antenna_pattern_idx": 1,
    "patternType": 1,
    "rotationAxis": 2,
    "nofApplicableTxAnt": 1,
    "nofApplicableRxAnt": 4,
    "applTxAntInd": [0],
    "applRxAntInd": [0,1,2,3],
    "gainCalcMethod": 1
}
antPatTable2 = {
    "antenna_pattern_idx": 2,
    "patternType": 1,
    "rotationAxis": 1,
    "nofApplicableTxAnt": 2,
    "nofApplicableRxAnt": 8,
    "applTxAntInd": [0,1],
    "applRxAntInd": [0,1,2,3,4,5,6,7],
    "gainCalcMethod": 1
}
antPatTable3 = {
    "antenna_pattern_idx": 3,
    "patternType": 1,
    "rotationAxis": 2,
    "nofApplicableTxAnt": 1,
    "nofApplicableRxAnt": 8,
    "applTxAntInd": [1],
    "applRxAntInd": [0, 1,2,3,4,5,6,7],
    "gainCalcMethod": 1
}

conf0={
    "diagTypeInfo": 1, # set 1 for ptu2 files WITH diagram type info, otherwise set 0.
    "targetAngle": 1,  # Show in target angle.
    "upsideDown": 0,# Set 1 for sensor upside down, otherwise set 0.
    "upsideDownInfo": 1,  # Set 1 for ptu2 files WITH upside down info, otherwise set 0.
    "tables": [antPatTable0,antPatTable1],

}
conf1={
    "diagTypeInfo": 0, # set 1 for ptu2 files WITH diagram type info, otherwise set 0.
    "targetAngle": 1,  # Show in target angle.
    "upsideDown": 0,# Set 1 for sensor upside down, otherwise set 0.
    "upsideDownInfo": 0,  # Set 1 for ptu2 files WITH upside down info, otherwise set 0.
    "tables": [antPatTable0,antPatTable1],


}
ptu2InputConfList=[conf0]

def start_data_evaluation(parseDataList,antpatLogger):

    ########### Script ###########
    # antPatHeader = collections.OrderedDict(antPatHeader)
    # # Save as YAML
    # with open('result.yml', 'w+') as yaml_file:
    #     yaml.dump(antPatHeader, yaml_file, default_flow_style=False)


    # Maximum number of data tables allowed
    MAX_NOF_DATA_TABLES = 8

    # Output directory path
    outputPath = aux.antPatOutputPath

    # Create directory if doesn't exist
    crDirStatus, crDirErrorString = aux.createDirectory(outputPath)

    # Setup logging configuration
    myLogger = logconfig.antPatLogger.logger
    errorStatus, errorSting = logconfig.antPatLogger.addFileHandler(logconfig.antPatLogFilePath, 'w+')

    if errorStatus == 1:
        myLogger.warning(errorSting)

    if not crDirStatus:
        myLogger.error(crDirErrorString)
        return

    myLogger.info('Starting.')

    # Clear the output folder
    clearDataStatus, clearDataString = aux.clearAll(aux.antPatOutputPath)
    if not clearDataStatus:
        myLogger.error(clearDataString)
        return

    # Set number of data tables to the maximum if not satisfied
    if portData["nofDataTables"]>MAX_NOF_DATA_TABLES:
        portData["nofDataTables"]=MAX_NOF_DATA_TABLES
        myLogger.info("Nof. Data Tables adjusted to the allowed maximum: "+str(MAX_NOF_DATA_TABLES)+'.')

    ## Manual mode
    if setup["auto"]==0:

        ## Setup initialisation

        # Initialisation of the list containing the setup data for files WITH and WITHOUT diagram type info.
        confList = []
        confError=0

        for ptu2InputConfIdx in range(len(ptu2InputConfList)):

            # Initialise configured data
            ptu2InputConf=dp.Ptu2InputConf(ptu2InputConfList[ptu2InputConfIdx],ptu2InputConfIdx)
            myLogger.info("Setup for PTU2 input configuration: " + ptu2InputConf.getStr() + '.')
            configuredData=dp.Setup_v2(setup,antPatHeader,ptu2InputConf,chirpConfig,parseDataList,MAX_NOF_DATA_TABLES,portData)

            if configuredData.errorStatus==0:
                confList.append(configuredData)
            else:
                myLogger.error("Invalid PTU2 input configuration: "+ configuredData.ptu2InputConf.getStr()+'.')

        # Set error if list is empty
        if len(confList) == 0:
            confError = 1

        ## Pattern generation and plot
        # Check if the setup list error occures
        if confError == 0:
            # Traverse through the setup list
            for confIdx in range(len(confList)):

                myLogger.info("Looking for PTU2 files: " + confList[confIdx].string + ".")

                # Check if the scheduled setup is valid
                if confList[confIdx].errorStatus == 0:

                    # # Traverse through the Antenna Type list
                    # for antTypeIdx in range(len(confList[confIdx].antennaTypesList)):

                    # Traverse through the angle setup list
                    for angSetupIdx in range(len(confList[confIdx].angleSetupAzElComb)):

                        # Traverse through the chirp list
                        for chirpIdx in range(len(confList[confIdx].confChirpList)):

                            myLogger.info("Table lookup scheduled, PTU input config.: "+ confList[confIdx].ptu2InputConf.getStr() +", Antenna Type, Diagram type and Angle setup comb.: " +
                                          confList[confIdx].angleSetupAzElComb[angSetupIdx].string + ', Chirp' + str(confList[confIdx].confChirpList[chirpIdx]) + '.')

                            # Initialise YAML table data container class for each Antenna Type, Angle setup, and Chirp comb.
                            yamlTable = dp.YAML_Table()
                            yamlTable.antennaType = confList[confIdx].angleSetupAzElComb[angSetupIdx].az.antennaType
                            yamlTable.chirp = confList[confIdx].confChirpList[chirpIdx]
                            yamlTable.azElCombSetup = confList[confIdx].angleSetupAzElComb[angSetupIdx]

                            # Traverse through the configured tables list
                            for tabIdx in range(len(confList[confIdx].confTablesList)):

                                myLogger.info("Lookig for data: " + confList[confIdx].ptu2InputConf.getStr()+", Antenna Type, Diagram type and Angle setup comb.: " +
                                              confList[confIdx].angleSetupAzElComb[angSetupIdx].string + ' - Chirp' + str(confList[confIdx].confChirpList[chirpIdx]) + \
                                              ' - Table' + str(confList[confIdx].confTablesList[tabIdx].antennaPatternIdx) + ' - ' +confList[confIdx].confTablesList[tabIdx].getStr())


                                if confList[confIdx].confTablesList[tabIdx].rotationAxis == 1:
                                    dataTable = dp.DataTable(
                                        confList[confIdx].angleSetupAzElComb[angSetupIdx].az,
                                        confList[confIdx].confTablesList[tabIdx],
                                        confList[confIdx].confChirpList[chirpIdx], parseDataList,
                                        confList[confIdx].ptu2InputConf)
                                else:
                                    dataTable = dp.DataTable(
                                        confList[confIdx].angleSetupAzElComb[angSetupIdx].el,
                                        confList[confIdx].confTablesList[tabIdx],
                                        confList[confIdx].confChirpList[chirpIdx], parseDataList,
                                        confList[confIdx].ptu2InputConf)


                                ## Plot initialisation ##
                                #  Close all the figure windows
                                plt.close("all")

                                # Create subplots and plot figure
                                fig, ax1 = plt.subplots()

                                # Invert axis for showing angle from right to left
                                plt.gca().invert_xaxis()

                                # Show plot grid
                                plt.grid(True)

                                # Set title to the graph figure
                                ax1.set_title("Antenna Patterns")

                                # Set y-axis label for specific general graph types
                                ax1.set_ylabel("Norm. Level [dB]")

                                # Set the x-axis label
                                ax1.set_xlabel('PTU Angle [°]')

                                # Streching the x axis data from end to end
                                ax1.autoscale(enable=True, axis='x', tight=True)

                                # Set plot x-axis label.
                                if confList[confIdx].ptu2InputConf.targetAngle == 1:
                                    ax1.set_xlabel('Target Angle [°]')

                                # Check if the Data Table object contains data for the input parameters.
                                if dataTable.errorStatus == 0:

                                    # Initialise the parameters for the angle and the average normalised level
                                    nofParsedFiles = len(dataTable.tableMeasDataList)
                                    angle = dataTable.angle
                                    avgNormLevelPerFileSumW = 0
                                    avgNormLevelW = 0

                                    # Traverse through each parsed PTU2 data cntained at the data table.
                                    for parFileIdx in range(nofParsedFiles):
                                        # Traverse through the configured Rx antennas.
                                        for rxAntIdx in range(len(dataTable.tableMeasDataList[parFileIdx].normLevelList)):
                                            # Get and plot the normalised level dB values for each Rx antenna.
                                            normLevelRxAntDb = \
                                            dataTable.tableMeasDataList[parFileIdx].normLevelList[rxAntIdx]
                                            ax1.plot(angle, normLevelRxAntDb, '.-')

                                        # Get the average normalised level from each Rx antenna, per PTU2 file.
                                        avgNormLevelPerFileW = dataTable.tableMeasDataList[
                                            parFileIdx].rxAntAvgNormLevelW
                                        avgNormLevelPerFileDb = dataTable.tableMeasDataList[
                                            parFileIdx].rxAntAvgNormLevelDb

                                        # Sum the average normalised level of eash PTU2 file for the purpose of calculating the average of it.
                                        avgNormLevelPerFileSumW = avgNormLevelPerFileSumW + avgNormLevelPerFileW

                                    # Average of the normalised level averages from each PTU2 file.
                                    avgNormLevelW = avgNormLevelPerFileSumW / nofParsedFiles

                                    # Convert the average norm level of all PTU2 files in dB, plot, and save figure.
                                    avgNormLevelDb = 10 * np.log10(avgNormLevelW)
                                    dataTable.avgNormLevelDb = avgNormLevelDb
                                    lines = ax1.plot(angle, avgNormLevelDb)
                                    plt.setp(lines, linestyle='--', linewidth='4', color='gold')
                                    figureTitle = dataTable.getStr()
                                    ax1.set_title(figureTitle)

                                    figurePath = os.path.join(outputPath, figureTitle)
                                    saveFig, saveFigStr = aux.saveFig(fig, figurePath)
                                    if saveFig:
                                        myLogger.info("Image generated: " + figureTitle)
                                    else:
                                        myLogger.warning(saveFigStr)

                                    # Append the table to the data table list in the YAML_Table object.
                                    validDataTable = copy.deepcopy(dataTable)
                                    yamlTable.dataTablesList.append(validDataTable)

                                else:
                                    # Data table doesnt contain data for the certain input parameters.
                                    myLogger.error("No available data for the scheduled Table found.\n")

                            # Append the the YAML_Table to the YAML tables list in the Setup object.
                            confList[confIdx].yamlTablesList.append(yamlTable)
                else:
                    # No data for the certain Setup found.
                    myLogger.error("No data for setup " + confList[confIdx].string + " found")

        else:
            # Incorrect configuration.
            myLogger.error("Incorrect configuration.")

    ## YAML Table generation

    # Check if configuration is correct.
    if confError==0:
        # Traverse through the setup list.
        for confIdx in range(len(confList)):

            myLogger.info("Looking for YAML tables data for setup " +confList[confIdx].string+ '.')

            # Check if the setup list is not empty.
            if confList[confIdx].errorStatus == 0:

                # # Traverse through the Antenna Type list.
                # for antTypeIdx in range(len(confList[confIdx].antennaTypesList)):

                # Traverse through the angle setup list.
                for angSetupIdx in range(len(confList[confIdx].angleSetupAzElComb)):

                    # Traverse through the chirp list.
                    for chirpIdx in range(len(confList[confIdx].confChirpList)):

                        # Check if the YAML_Table list for certain Setup is not empty.
                        if len(confList[confIdx].yamlTablesList)!=0:

                           # Traverse through the YAML_Table list.
                            for yamlTableIdx in range(len(confList[confIdx].yamlTablesList)):

                                # Check if the actual YAML_Table matches for the iteration with certain Antenna Type, Angle setup, and Chirp.
                                if confList[confIdx].yamlTablesList[yamlTableIdx].chirp==confList[confIdx].confChirpList[chirpIdx] and confList[confIdx].yamlTablesList[yamlTableIdx].azElCombSetup ==confList[confIdx].angleSetupAzElComb[angSetupIdx]:

                                    myLogger.info("Generating YAML file: "+confList[confIdx].string +", Antenna Type, Diagram type and Angle setup comb.: " + confList[confIdx].angleSetupAzElComb[angSetupIdx].string + ', Chirp' + str(confList[confIdx].confChirpList[chirpIdx]) + '.')

                                    # Assign JSOn file title and reorder data tables list which contains only valid data tables.
                                    yamlFileTitle =confList[confIdx].string +' - '+confList[confIdx].angleSetupAzElComb[angSetupIdx].string + ' - ' + 'Chirp' + str(confList[confIdx].confChirpList[chirpIdx]) + '.yaml'

                                    yamlTable=confList[confIdx].yamlTablesList[yamlTableIdx]
                                    yamlTable.reorderDataTablesList()
                                    yamlTable.yamlFileTitle = yamlFileTitle

                                    # Assign the Antenna Patterns header to the YAML_Table object and adjust the nof. data tables and rfeGeneration.
                                    outputDict = copy.deepcopy(confList[confIdx].antPatHeader)
                                    portData1 =copy.deepcopy(confList[confIdx].portData)
                                    portData1["nofDataTables"] = len(yamlTable.dataTablesList)
                                    outputDict["rfeGeneration"]=confList[confIdx].yamlTablesList[yamlTableIdx].antennaType
                                    yamlTable.outputDict = outputDict
                                    yamlTable.portData = portData1

                                    myLogger.info("Nof. data tables adjusted to the number of available data tables: " + str(yamlTable.portData["nofDataTables"]) + '.')

                                    # Check if the YAML_Table contains data tables
                                    if len(yamlTable.dataTablesList)!=0:

                                        # Traverse through the data tables list
                                        for dataTableIdx in range(len(yamlTable.dataTablesList)):

                                            # Reorder angle and avg normalised level data for the purpose of presenatation.
                                            dataTable=yamlTable.dataTablesList[dataTableIdx]

                                            upsideDown=dp.getUpsideDown(dataTable.tableMeasDataList[0].parseData, dataTable.ptu2InputConf)
                                            angleData=dataTable.angle
                                            normLevelData = dataTable.avgNormLevelDb
                                            # revOrderAngle = angleData[::-1]
                                            if dataTable.ptu2InputConf.targetAngle==1 and upsideDown==0:
                                                angleData=angleData[::-1]
                                                normLevelData = normLevelData[::-1]

                                            angleData=angleData.tolist()
                                            # revOrderAngle = revOrderAngle.tolist()

                                            normLevelData = normLevelData.tolist()
                                            # revOrderNormLevel = normLevelData[::-1]
                                            # revOrderNormLevel = revOrderNormLevel.tolist()

                                            # Append the data tables dict to the list.
                                            yamlTable.dataTablesDictList.append({

                                                #"antenna_pattern_idx": dataTable.configTable.antennaPatternIdx,
                                                "patternType": dataTable.configTable.patternType,
                                                "rotationAxis": dataTable.configTable.rotationAxis,
                                                #"nofApplicableTxAnt": dataTable.configTable.nofApplicableTxAnt,
                                                #"nofApplicableRxAnt": dataTable.configTable.nofApplicableRxAnt,
                                                "applTxAntIdx":{"start": dataTable.configTable.applTxAntInd},
                                                "applRxAntIdx":{"start": dataTable.configTable.applRxAntInd},
                                                #"gainCalcMethod": dataTable.configTable.gainCalcMethod,
                                                #"nofAngleValues": dataTable.nofAngleValues,
                                                "angle":{"start": angleData},
                                                "normGainDb":{"start": normLevelData}
                                            })

                                        #create meta dict
                                        yamlTable.outputDict.update({"meta" : 
                                                                    { "version":1,
                                                                    "housingGeneration": 7,
                                                                    "housingModification": 7,
                                                                    "rfeGeneration": confList[confIdx].yamlTablesList[yamlTableIdx].antennaType,
                                                                    "rfeModification": 0,
                                                                    "source": "measurement"}})

                                        # Form a dict of the data tables
                                        dataTablesDict = {"patterns":{"start": yamlTable.dataTablesDictList}}

                                        # Update port data with data tables
                                        portData1.update(dataTablesDict)

                                        # Form a dict of the data tables
                                        portDataDict = {"data": portData1}

                                        # Assemble the header part with the port data.
                                        yamlTable.outputDict.update(portDataDict)
                                        

                                        outputPath = os.path.join(aux.antPatOutputPath, yamlTable.yamlFileTitle)

                                        # myLogger.info('YAML file generated: ' + yamlTable.yamlFileTitle + '.')

                                        # Save as YAML
                                        yamlComment = "#patternType - Antenna pattern type (0: invalid; 1: TX - RX; 2: TX; 3: RX).\n" + \
                                                      "#rotationAxis - Antenna rotation axis (0: invalid; 1: z, azimuth; 2: y, elevation).\n" + \
                                                      "#applTxAntIdx - [1] Indices of the TX antennas the pattern is applicable to.\n" + \
                                                      "#applRxAntIdx - [1] Indices of the RX antennas the pattern is applicable to.\n" + \
                                                      "#angle - [deg] Antenna pattern angle values.\n" + \
                                                      "#normGainDb - [dB] Normalized antenna pattern gain values.\n"
                                                      #"#antenna_pattern_idx - Index of the data table.\n" + \
                                                      #"#nofApplicableTxAnt - [1] Number of TX antennas the pattern is applicable to.\n" + \
                                                      #"#nofApplicableRxAnt - [1] Number of RX antennas the pattern is applicable to.\n" + \
                                                      #"#gainCalcMethod - Method of data calculation (0: invalid, 1: step-wise, 2: interpolation).\n" + \
                                                      #"#nofAngleValues - [1] Number of angle values.\n" + \

                                        with open(outputPath, 'w+') as yaml_file:
                                            yaml_file.write(yamlComment)
                                            yaml.dump(yamlTable.outputDict, yaml_file, default_flow_style=False,sort_keys=False)

                                    else:
                                        # No data tables found for the certain configuration
                                        myLogger.error('No data tables for YAML file found, Antenna Type: '+str(confList[confIdx].yamlTablesList[yamlTableIdx].antennaType) +', Diagram type and Angle setup comb.: '+confList[confIdx].angleSetupAzElComb[angSetupIdx].string + ', Chirp' + str(confList[confIdx].confChirpList[chirpIdx]) + '.')

                        else:
                            # No YAML tables found for the certain Setup.
                            myLogger.error("No YAML tables for setup " + confList[confIdx].string + ' available.')
    #

    # Copying the LogFile to the output directory
    logFileOutputPath = os.path.join(aux.antPatOutputPath, logconfig.LogFileNames.AntennaPatterns.value)
    status, errorSting =aux.copyFile(logconfig.antPatLogFilePath, logFileOutputPath)
    if not status:
        myLogger.warning(errorSting)

    loadDataLogFileOutputPath = os.path.join(aux.antPatOutputPath, logconfig.LogFileNames.MainLoadData.value)
    status,errorSting = aux.copyFile(logconfig.antPatLogFilePath, loadDataLogFileOutputPath)
    if not status:
        myLogger.warning(errorSting)
        
if(bUseGui):
    pass

else:

    # import antpat_logconfig
    antpatLogger = logconfig.antPatLogger
    antpatLog = logconfig.antPatLogger.logger
    errorStatus, errorSting = logconfig.antPatLogger.addFileHandler(logconfig.antPatLogFilePath, 'w+')

    if errorStatus ==1:
        antpatLog.warning(errorSting)
    antpatLog.info('Starting.')
    # Load Data by absolute path
    antpatLog.info('Loading data.')
    loadData = ld.LoadData(absPath,setup["useSensorMuster"],setup["sensorMusterList"])

    if loadData.invalidPath != 1:
        # myLogger.info("Num. of valid files parsed: " + str(loadData.validFilesCounter) + ".")
        parseDataList = loadData.parseDataList
        start_data_evaluation(parseDataList,antpatLogger)
    else:
        antpatLog.error(loadData.invalidPathString)
